
;+ ***********************************************************************
; NAME:
;	MENUTRACEISOPH
;
; PURPOSE:
;	Cette procedure contient les WIDGETS pour le trace de contours
;	des fichiers 1 dimension NRH, et calcul de positions de centroids 
;	"Button" CONTOURS du menu principal
;
; CATEGORY:
;	NRH1 Menu
;
; CALLING SEQUENCE:
;	MENUTRACEISOPH
;
; INPUTS:
;	Non
;
; KEYWORD PARAMETERS:
;	Non
;
; OUTPUTS:
;	Non
;
; COMMON BLOCKS:
;	NRH1_GLOBAL
;	NRH1_HELIO
;
; MODIFICATION HISTORY: (bonmartin@obspm.fr)
;	16/11/98 adapte du logiciel XHELIO
; Modifications:
;		24/11/97 Cadrage du trace pour obtenir un remplissage maximum
;				de la page
;
;		10/02/98 Trace des profils avec curseur glissant
;		02/04/99 Correction de la fenetre du trace des positions
;                        pour la sortie POSTSCRIPT
;               11/2006  trace des isoph en coordonnees heliographiques 
;-*******************************************************************

PRO MENUTRACEISOPH_EVENT, ev


ON_ERROR,2
common NRH1_GLOBAL,nomfich,fichinfo
common NRH1_HELIO,mainstruct

; on charge les variables locales a isophote
stash = WIDGET_INFO ( ev.top, /CHILD )
WIDGET_CONTROL, stash, GET_UVALUE = state

; ***** Activation/desactivation du choix des positions *****  

IF (ev.id EQ state.allpos) THEN BEGIN
   IF (ev.value EQ 2) THEN BEGIN
      WIDGET_CONTROL,state.numpos,SENSITIVE = 1
   ENDIF ELSE BEGIN
      WIDGET_CONTROL,state.numpos,SENSITIVE = 0
   ENDELSE
ENDIF

; ***** Lecture du fichier de positions ***** 

IF (ev.id EQ state.fposition) THEN BEGIN
   WIDGET_CONTROL, state.fposition, GET_VALUE = fpos
   err = newlitentete(1,fpos(0),file)
   IF (err EQ 1) THEN RETURN
   Nbpos = File.nbpix(1)/File.facbloc(1)
   tabpos = intarr(nbpos)
   tabpos(*) = 0
   oldwidget=state.numpos
   state.numpos = CW_GRID(state.base_position,indgen(nbpos),$
                          TITLE = 'positions')
   WIDGET_CONTROL,state.allpos,GET_VALUE=result
   IF (result EQ 1) THEN $
      WIDGET_CONTROL, state.numpos, SENSITIVE = 0
   WIDGET_CONTROL,oldwidget,/destroy
   WIDGET_CONTROL, state.base_position,/realize
   WIDGET_CONTROL, state.numpos, SET_VALUE = tabpos
ENDIF

; ***** Lecture du fichier d'images *****

IF (ev.id EQ state.files(0)) THEN BEGIN
   WIDGET_CONTROL, state.files(0), GET_VALUE = fichier
   err = newlitentete(1,fichier(0),file)
   IF (err EQ 1) THEN BEGIN 
	R = WIDGET_MESSAGE('Read ERROR on file '+ fichier(0))
   RETURN & END
;	Mise hors service des widgets "curseur/zoom" et "calcul positions"
;	apres la lecture (et avant le trace) du fichier d'images:
   WIDGET_CONTROL, state.base_decomp, SENSITIVE = 0
   WIDGET_CONTROL, state.base_zoom, SENSITIVE = 0

   WIDGET_CONTROL, state.maxvalue, SET_VALUE = file.valmax
   WIDGET_CONTROL, state.timestep, GET_VALUE = deltat0
   deltat = file.paspix(1)
;... Controle du widget heures: Ne pas changer si heure deja mise   
   WIDGET_CONTROL, state.heures, GET_VALUE = test
   heurdeb = hms(test(0))
   heurfin = hms(test(1))
   IF (heurfin LE heurdeb OR heurfin EQ 0 OR $
		ABS(deltat-deltat0) GT 0.05*deltat) THEN BEGIN
	hdeb=MSH(file.heurdeb)
	hfin=MSH(file.heurfin)
	WIDGET_CONTROL, state.heures, SET_VALUE = [hdeb,hfin]
	WIDGET_CONTROL, state.timestep, SET_VALUE = deltat
   ENDIF
;... Controle du widget canaux: Ne pas changer si selection existe deja
   valres = file.texte.val(6)
   IF valres EQ "EST-OUEST" OR Valres EQ 'EAST_WEST' THEN $
	WIDGET_CONTROL, state.canauxew, GET_VALUE = test $
		ELSE $
			WIDGET_CONTROL, state.canauxsn, GET_VALUE = test
   cdeb0 = FIX(test(0))
   cfin0 = FIX(test(1))
   cdeb=fix(file.valorg(0))
; pour passages des unites de canaux aux unites de rayons solaires
    cfin=fix( file.valorg(0) + file.paspix(0)*file.nbpix(0)  )
;   cfin=fix( file.valorg(0) + file.paspix(0)*file.nbpix(0) - 1 )
   IF (cdeb GT cdeb0 OR cfin LT cfin0) THEN BEGIN
	   IF valres EQ "EST-OUEST" OR Valres EQ 'EAST_WEST' THEN $
		WIDGET_CONTROL, state.canauxew, SET_VALUE = [cdeb,cfin] $
		ELSE $
		WIDGET_CONTROL, state.canauxsn, SET_VALUE = [cdeb,cfin]
   ENDIF
ENDIF

; Pour les fichiers FITS choix EW NS
IF Ev.id EQ State.Id_ewns THEN $
	Mainstruct.array = ev.value

;-----------------------------------------------------------------------------
; ***** Action demandee par bouton de controle *****

IF ( ev.id EQ state.bgroup_control ) THEN BEGIN
  ; ******** update ********* on recupere les variables globales
    IF ( ev.value EQ 0 ) THEN BEGIN
	WIDGET_CONTROL, state.files(0), GET_VALUE = fichier
	err = newlitentete(1,fichier(0),file)
	IF (err EQ 1) THEN RETURN
 	hdeb=MSH(file.heurdeb)
	hfin=MSH(file.heurfin)
	WIDGET_CONTROL, state.heures, SET_VALUE = [hdeb,hfin]
	cdeb=fix(file.valorg(0))
;	cfin=fix( file.valorg(0) + file.paspix(0)*file.nbpix(0) - 1 )
; pour passages des unites de canaux aux unites de rayons solaires
	cfin=fix( file.valorg(0) + file.paspix(0)*file.nbpix(0)  )
stop

	valres = file.texte.val(6)
	IF valres EQ "EST-OUEST" OR valres EQ 'NORTH-SOUTH' THEN $
		WIDGET_CONTROL, state.canauxew, SET_VALUE = [cdeb,cfin] $
		ELSE $
		WIDGET_CONTROL, state.canauxsn, SET_VALUE = [cdeb,cfin]

     ENDIF
  ; ******* quit *********
     IF ( ev.value EQ 1 ) THEN BEGIN
         WIDGET_CONTROL, stash, SET_UVALUE = state
         WIDGET_CONTROL, ev.top,/DESTROY
         RETURN
     ENDIF
  ; ******* trace isoph (2) ou niveaux de gris (3), les deux (4) ou 3D (5)*********
    IF ( ev.value GE 2 ) THEN BEGIN
      ; *******   On recupere  toutes les valeurs des widgets *********
        WIDGET_CONTROL,state.maxvalue, GET_value = maxvalue
	WIDGET_CONTROL,state.niveau, GET_value = niveau
        WIDGET_CONTROL, state.nlevels, GET_VALUE = nlevels
        WIDGET_CONTROL, state.heures, GET_VALUE = heures
	WIDGET_CONTROL, state.files(0), GET_VALUE = fichier

	valres = MOTCLE(1, 'LABEL')
	IF valres EQ "EST-OUEST" OR Valres EQ 'EAST-WEST'THEN $
	WIDGET_CONTROL, state.canauxew, GET_VALUE = canaux $
		ELSE $
			WIDGET_CONTROL, state.canauxsn, GET_VALUE = canaux
        WIDGET_CONTROL, state.files(0), GET_VALUE = fichier      
        WIDGET_CONTROL,state.bgroup,GET_VALUE=choix_qualifiers

        fonction = state.fonction    
        intens=Choix_Qualifiers(0)
        polar=Choix_Qualifiers(1)
        tek=Choix_Qualifiers(2)
        noerase = Choix_Qualifiers(3)

        IF (polar EQ 0) AND (intens EQ 0) THEN $
        RETURN
        IF (polar EQ 1) AND (intens EQ 1) THEN BEGIN
           polar=0
           intens=1
        ENDIF
 
        err = newlec1d(2,a,sa,FICHIER=fichier(0))
        IF (err EQ 1) THEN RETURN

;     on utilise que la premiere moitiee de la table des couleurs
;     pour les isophotes >> possibilite de faire un degrade
;     on garde le reste pour superposer les positions

        couleurs = intarr(nlevels(0))
        pas = (!D.table_size/2-20)/(nlevels(0)+1)
        FOR inc = 0,nlevels(0)-1 DO BEGIN
           couleurs(inc)= 20 + (inc+1) * pas
        ENDFOR 
    
	lun=2

;  ***** AFFICHAGE DES ISOPHOTES ******
;     choix de fenetre a utiliser
	IF (KEYWORD_SET(noerase)) THEN BEGIN
          win = CALL_FUNCTION('get_oldwindow',5,8)
	ENDIF ELSE win = get_window(5,8,TITLE='isophote')

;  Definition de l'emplacement du cadre / des cadres modif 21/3/96 KLK

; Definition des coins inferieur gauche & superieur droit du cadre

        x0 = 0.11 & x1 = 0.75 & x1laser = 0.89 & y0 = 0.1 & y1 = 0.9
        !p.position(0) = x0 & !p.position(2) = x1
	IF( !P.MULTI(2) EQ 0) THEN BEGIN
		distcad = y1-y0	; Distance between successive frames
		!p.position(3) = $
		   y1 - 0.21*distcad & !P.position(1) = y0 + 0.21*distcad
	ENDIF ELSE BEGIN
	     distcad = (y1-y0)/!P.MULTI(2);Distance between successive frames
		IF(!P.MULTI(1) GT 1) THEN $
			Res=WIDGET_MESSAGE('WARNING: many frames')
		IF(!P.MULTI(0) EQ 0) THEN ymaxc = y1 - .21*distcad ELSE $
			ymaxc = y0 + (!P.MULTI(0)-.21)*distcad
		yminc = ymaxc - .58*distcad
		!p.position(1) = yminc & !P.position(3) = ymaxc
	ENDELSE

    IF (mainstruct.laser EQ 1) THEN BEGIN
        NRH1_postscript,/ON
	!p.position(2) = x1laser
        traceisoph,a,lun,nlevels=nlevels(0),$
            HEURES=heures,CANAUX=canaux, $
            POLAR=polar,INTENS=intens,$
            TEK=tek,noerase=noerase, CHARSIZE = 2, $
            NIV=niveau(0),$
            C_COLORS=couleurs, FONCTION=fonction, GRAY = ev.value - 2
	!p.position(2) = x1
        NRH1_postscript,/OFF
    ENDIF

    traceisoph,a,lun,nlevels=nlevels(0),$
            HEURES=heures,CANAUX=canaux, $
            POLAR=polar,INTENS=intens,$
            TEK=tek,noerase=noerase, CHARSIZE = 1,$
            NIV=niveau(0),$
            C_COLORS=couleurs, FONCTION=fonction, GRAY = ev.value - 2

;*** Identification du fichier de positions a partir du nom du fichier 
;    d'images
    IF (Choix_Qualifiers(1) EQ 1) THEN valsto = 'V' ELSE valsto = 'I'
    nslash = RSTRPOS(fichier(0),'/')
    fpos = STRMID(fichier(0), 0, nslash) + '/p' + valsto + $
		 STRMID(fichier(0), nslash+1, STRLEN(fichier(0))-nslash)

; Modification pour fichier images FITS
		S= STR_SEP(Fpos, '.')
		Sz= SIZE(s)
		IF S(1) EQ 'fts' THEN $
 		    IF Mainstruct.array EQ 0 THEN FPOS = S(0)+'.few'$
			ELSE FPOS = S(0)+'.fns'

    WIDGET_CONTROL, state.fposition, SET_VALUE = fpos
    WIDGET_CONTROL,state.base_zoom, SENSITIVE=1
    WIDGET_CONTROL,state.base_decomp, SENSITIVE=1

    state.xrange = canaux
    state.yrange = [HMS(heures(0)),HMS(heures(1))]
    state.debutemp = fichinfo(lun).heurdeb
	state.pastempo = FichInfo(lun).paspix(1)

	mainstruct.heures=heures
	mainstruct.canaux=canaux         
    ENDIF
ENDIF

; ---------------------------------------------------------------------------

; groupe de boutons non exclusifs

; ***** Trace des positions sur la carte d'isophotes ***
IF ( ev.id EQ state.tracepos ) THEN BEGIN
	IF (state.xrange(0) EQ 0 AND state.xrange(1) EQ 0) THEN RETURN

	canrange = [state.xrange(1), state.xrange(0)]
	x0 = 0.11 & x1 = 0.75  & x1laser = 0.89 & y0 = 0.1 & y1 = 0.9
	!p.position(0) = x0 & 	!p.position(2) = x1
 	multi = !p.multi(0)
	!p.multi(0) = multi + 1
	IF( !P.MULTI(2) EQ 0) THEN BEGIN
		distcad = y1-y0	; Distance between successive frames
		!p.position(3) = y1 - 0.21*distcad 
		!P.position(1) = y0 + 0.21*distcad
	ENDIF ELSE BEGIN
		distcad = (y1-y0)/!P.MULTI(2)
		IF(!P.MULTI(1) GT 1) THEN PRINT, $
				'Probleme def. des cadres: plusieurs colonnes'
		IF(!P.MULTI(0) EQ 0) THEN ymaxc = y1 - .21*distcad $
				ELSE ymaxc = y0 + (!P.MULTI(0)-.21)*distcad
		yminc = ymaxc - .58*distcad
			!p.position(1) = yminc & !P.position(3) = ymaxc
	ENDELSE

	!x.window(0) = !p.position(0) & !y.window(0) = !p.position(1)
	!x.window(1) = !p.position(2) & !y.window(1) = !p.position(3)

	WIDGET_CONTROL, state.fposition, GET_VALUE = fpos
             err = newlecp1d(4,b,sb,FICHIER=fpos(0), /oldmode)
             IF (err EQ 1) THEN RETURN

	WIDGET_CONTROL, state.allpos, GET_VALUE = allpos

	IF ( allpos EQ 2 ) THEN BEGIN ; trace des positions selectionnees
		WIDGET_CONTROL, state.numpos, GET_VALUE = indpos
		j=0
;               pascol=(!D.table_size-80)/((size(b))(1)-1)
		pascol=(!D.table_size-80)/(size(b))(1)
		FOR i = 0,(size(b))(1)-1 DO BEGIN 
		    IF (indpos(i) EQ 1) THEN BEGIN
                        win=get_oldwindow(5,8)    
                        couleur=80+j*pascol
                        IF (mainstruct.laser EQ 1) THEN BEGIN
                            NRH1_postscript,/on
			    !p.position(2) = x1laser
                            err = traceposiso(b,sb,i,state.yrange,canrange,$
                                     state.pastempo,$
                                     COLOR=couleur,$
                                     /INDICE)
			    !p.position(2) = x1
                            NRH1_postscript,/off
                            IF err EQ 1 THEN RETURN
                         ENDIF
                            err = traceposiso(b,sb,i,state.yrange,canrange,$
                                     state.pastempo,$
                                     COLOR=couleur,$
                                     /INDICE)
                        IF err EQ 1 THEN RETURN                        
                        j = j + 1
                    ENDIF
		ENDFOR
            ENDIF ELSE BEGIN ; trace de toutes les positions
;            pascol=(!D.table_size-80)/((size(b))(1)-1)
               pascol=(!D.table_size-80)/(size(b))(1)
               FOR i = 0,(size(b))(1)-1 DO BEGIN
                    win=get_oldwindow(5,8)    
                    couleur=80+i*pascol
                    IF (mainstruct.laser EQ 1) THEN BEGIN
                        NRH1_postscript,/on
			!p.position(2) = x1laser
                            err = traceposiso(b,sb,i,state.yrange,canrange,$
                                 state.pastempo,$
                                 COLOR=couleur,$
                                   /INDICE)
			!p.position(2) = x1
                        NRH1_postscript,/off
                        IF err EQ 1 THEN RETURN
                     ENDIF     
                            err = traceposiso(b,sb,i,state.yrange,canrange,$
                                 state.pastempo, $
                                  COLOR=couleur,$
                                      /INDICE)
                    IF err EQ 1 THEN RETURN
               ENDFOR
            ENDELSE
	!P.MULTI(0)=!P.MULTI(0)-1	; ajout klk 22/10/97 
ENDIF

; ***** Recherche de positions/decomposition des images *****:
; Aide en ligne: 
IF ( ev.id EQ state.aidepos ) THEN BEGIN
	aide, SUJET="recherche", message
	txt = WIDGET_MESSAGE(message, /INFORMATION, $
		TITLE = 'Extract individual sources')
ENDIF

IF ( ev.id EQ state.bg_decomp) THEN BEGIN ; choix de la fonction a ajuster
    state.txt_decomp = ev.index
ENDIF

IF ( ev.id EQ state.rechpos ) THEN BEGIN
     message = "Delimit a rectangle (time, channel) around the source"
     WIDGET_CONTROL, state.txt_message, SET_VALUE=message, /APPEND
     result=NRH1_getzone()

     IF N_ELEMENTS(result) EQ 1 THEN RETURN
     IF result(1) LT result(3) THEN $
     	canaux = [result(1), result(3)] ELSE canaux = [result(3), result(1)]
     temps = 0.5*(result(0) + result(2))
     lun=2

; *** Selection des parametres de recherche
;	WIDGET_CONTROL, state.bg_decomp
;	itype = WIDGET_INFO(state.bg_decomp, /droplist_select)
	itype = state.txt_decomp
	fittype = intarr(4) & fittype(itype) = 1
	WIDGET_CONTROL, state.txt_decompcan, GET_VALUE = gliss 
	WIDGET_CONTROL, state.txt_decompsig, GET_VALUE = poids
	WIDGET_CONTROL, state.txt_decompnbr, GET_VALUE = nposmax
	WIDGET_CONTROL, state.txt_decompeca, GET_VALUE = deltacan

; *********************************************************************
	WIDGET_CONTROL, state.files(0), GET_VALUE = fichier
        err = newlec1d(2,a,sa,FICHIER=fichier(0))
        IF (err EQ 1) THEN RETURN
	WIDGET_CONTROL,state.bgroup,GET_VALUE=choix_qualifiers
        polar=Choix_Qualifiers(1)
        WIDGET_CONTROL, state.heures, GET_VALUE = heures
     followpos1d,a, lun, gliss, poids, nposmax, Polar=Polar, TEMPS=temps, $
	HEURES=heures, CANAUX=canaux, DELTACAN=deltacan, $
	SIMPLE=fittype(0), CONST=fittype(1),$
	DROITE=fittype(2), PARABOLE = fittype(3), tabcoeff, numd, numf, ierr
;
	IF(ierr NE 0) THEN BEGIN
;		message = "Error (code  " + $
;		STRCOMPRESS(string(ierr)) + ") $
		message = "No source found in chosen interval" + $
		"  (not enough data points to fit). " 
	WIDGET_CONTROL, state.txt_message, SET_VALUE=message, /APPEND
                message = "Try to interpolate image with more points."
	WIDGET_CONTROL, state.txt_message, SET_VALUE=message, /APPEND
	message = "    field [" + $
		STRCOMPRESS(canaux(0)) + "," + STRCOMPRESS(canaux(1)) + "]" + $
		",  time: " + STRCOMPRESS(MSH(temps))
	WIDGET_CONTROL, state.txt_message, SET_VALUE=message, /APPEND
	ENDIF ELSE BEGIN
		message = "First scan: " + STRCOMPRESS( string(numd) ) + $
			", last scan: " + STRCOMPRESS(string(numf))
		WIDGET_CONTROL, state.txt_message, SET_VALUE=message, /APPEND
; ----Ecriture et trace de controle ------------------------------------------ 
		hdeb=fichinfo(lun).heurdeb	; heure debut fichier d'images
		hfin=fichinfo(lun).heurfin	; heure fin
		nbpts = numf-numd+1	; nbre d'enregistrements de la source
		nbmaxpt = nbpts
		npos = 1		; nbre de positions a ecrire
		idtms = LONG( fichinfo(lun).paspix(1))	; intervalle temps [ms]
		xmin = MIN(tabcoeff(0,*), MAX=xmax)
		trp = fltarr(5, nbmaxpt, npos)

		FOR num = 0, nbpts-1 DO BEGIN
			trp(0, num, npos-1) = numd + num
			trp(1, num, npos-1) = tabcoeff(1, num)	; canal
			ampli = tabcoeff(0, num) & fwhm = tabcoeff(2, num)
			trp(2, num, npos-1) = ampli	; amplitude
			trp(3, num, npos-1) = 1.06446 * ampli * fwhm	; flux
			trp(4, num, npos-1) = fwhm	; largeur
		ENDFOR

; *** Ecriture d'un nouveau fichier de position ou fusion avec un ancien:
		valfreq = MOTCLE(lun, 'FREQ')
		valres = MOTCLE(lun, 'LABEL')
		WIDGET_CONTROL, state.fposition, GET_VALUE = fpos
; Modification pour fichier images FITS
		S= STR_SEP(Fpos, '.')
		Sz= SIZE(s)
		IF S(1) EQ 'fts' THEN $
		    IF Mainstruct.array EQ 0 THEN FPOS = S(0)+'.few'$
			ELSE FPOS = S(0)+'.fns'
; rempace .fts pour les fichiers FITS

		itete = [1, heures(0), nbpts]
; *** Fichier de positions I ou V ? ***
	IF (Choix_Qualifiers(1) EQ 1) THEN $
	  chrec_cle=['INDEX','CHANNEL','V-AMPLITUDE','V-FLUX','DIAMETRE']$
	ELSE $
	  chrec_cle=['INDEX','CHANNEL','I-AMPLITUDE','I-FLUX','DIAMETRE']
	nbrec =  N_ELEMENTS(chrec_cle)
	c2idate, fichinfo(lun).date, idate
		SZ = SIZE(FINDFILE(Fpos))
	IF SZ(0) EQ 0 THEN $
			ECRPOS, xmin, xmax, idtms, hdeb, hfin, idate,$
				valfreq, valres, nbrec, chrec_cle,$
				fpos, npos, nbmaxpt, itete, trp $
	ELSE $
			FUSIONPOS1D, xmin, xmax, idtms, hdeb, hfin, idate,$
				valfreq, valres, nbrec, chrec_cle,$
				fpos, npos, nbmaxpt, itete, trp

; ----Fin ecriture-----------------------------------------------------------

		canrange = [state.xrange(1), state.xrange(0)]
		x0 = 0.11 & x1 = 1. & y0 = 0.1 & y1 = 0.9
		!p.position(0) = x0 & !P.position(2) = x1 

		multi = !p.multi(0)
		!p.multi(0) = multi + 1
		IF( !P.MULTI(2) EQ 0) THEN BEGIN
			distcad = y1-y0	; Distance between successive frames
			!p.position(3) = y1 - 0.21*distcad 
			!P.position(1) = y0 + 0.21*distcad
		ENDIF ELSE BEGIN
			distcad = (y1-y0)/!P.MULTI(2)
			IF(!P.MULTI(1) GT 1) THEN $
				Res=WIDGET_MESSAGE('WARNING: many frames')
			IF(!P.MULTI(0) EQ 0) THEN ymaxc = y1 - .21*distcad $
				ELSE ymaxc = y0 + (!P.MULTI(0)-.21)*distcad
			yminc = ymaxc - .58*distcad
			!p.position(1) = yminc & !P.position(3) = ymaxc
		ENDELSE

		x0 = 0.11 & x1 = 0.75  & x1laser = 0.89 & y0 = 0.1 & y1 = 0.9
		!p.position(0) = x0 & 	!p.position(2) = x1
		!x.window = [x0, x1] 
		!y.window(0) = !p.position(1) & !y.window(1) = !p.position(3)
		provtracepos, heures, tabcoeff(1,*), state.yrange, canrange
		!p.multi(0) = multi
	ENDELSE
ENDIF
; *** fin modif

; **** Selection d'une fonction pour le pas des niveaux *****
IF ( ev.id EQ state.menu) THEN BEGIN
	IF ev.index EQ 3 THEN BEGIN
		aide, SUJET="choix_isoph", message
		txt = WIDGET_MESSAGE(message, /INFORMATION, $
			TITLE = 'Selection of contour levels')
	ENDIF ELSE BEGIN
		state.fonction = ev.index 
	ENDELSE
ENDIF

; *** Determination des coordonnees d'un point avec le curseur (CURSEUR) ***
IF ( ev.id EQ state.curseur ) THEN BEGIN
	message = "Click on the point to be determined"
	WIDGET_CONTROL, state.txt_message, SET_VALUE=message, /APPEND
	result=NRH1_getzone(SINGLE=1)
	message = "   Time: " + MSH(result(0)) + ", channel: " + $
		STRCOMPRESS(STRING(result(1)))
	WIDGET_CONTROL, state.txt_message, SET_VALUE=message, /APPEND
ENDIF

IF ( ev.id EQ state.proftemp OR ev.id EQ state.profcan) THEN BEGIN
	message = "Choose the channel or instant of the plot (cursor)"
	WIDGET_CONTROL, state.txt_message, SET_VALUE=message, /APPEND
	result=NRH1_getzone(SINGLE=1)
	message = "   Time: " + MSH(result(0)) + ", channel: " + $
		STRCOMPRESS(STRING(result(1)))
	WIDGET_CONTROL, state.txt_message, SET_VALUE=message, /APPEND
; Memorisation des parametres de la fenetre des isophotes
	winisoph = get_oldwindow(5,8)
	winprof = winisoph - 3
	isotrans = [!X.S, !Y.S]	; mise en memoire de la transf. DATA/DEVICE
; Selection du profil temporel + trace 
	WIDGET_CONTROL, state.files(0), GET_VALUE = fichier
;	intensite = 1
	lun=2
        WIDGET_CONTROL,state.bgroup,GET_VALUE=choix_qualifiers
        intens=Choix_Qualifiers(0)
        polar=Choix_Qualifiers(1)
 	IF (ev.id EQ state.proftemp) THEN BEGIN
		win=get_window(winprof, winprof, TITLE='Amplitude')
	        WIDGET_CONTROL, state.heures, GET_VALUE = heures
		canaux = [result(1), result(1)]
	        err = newlec1d(lun,a,sa,FICHIER=fichier(0))
        	IF (err NE 1) THEN calcflux, a, lun, $
			 HEURES=heures, CANAUX=canaux, temps, FLUX
		IF (intens NE 0 OR (SIZE(flux))(0) EQ 1) THEN BEGIN
			iopt = 0 
			ytitle = "Densit de flux (I, "
		ENDIF ELSE BEGIN
			iopt=1
			ytitle = "Densit de flux (V, "
		ENDELSE
		ytitle = ytitle + "canal " $
			+ STRCOMPRESS(STRING(canaux(0))) + ")"
		trace1d, temps, flux(*, iopt), YTITLE = ytitle 
	ENDIF ELSE BEGIN
		win=get_window(winprof, winprof, TITLE='Profil')
	        err = newlec1d(lun,a,sa,FICHIER=fichier(0))
        	IF (err NE 1) THEN prof1d,a,lun, HEURE = MSH(result(0)), $
                   POLAR=polar,INTENSITE=intens,$
                   NOERASE=noerase

	ENDELSE
; Retablissement de la fenetre des isophotes et de sa transf. DATA/DEVICE 
	WSET, winisoph
	!X.S = isotrans(0:1) & !Y.S = isotrans(2:3)
ENDIF

; **** Profil glissant (SLID. SCAN) *****
IF ( ev.id EQ state.profcanslid) THEN BEGIN
	message = $
	"Slide along the time axis, exit with CENTRAL button"
	WIDGET_CONTROL, state.txt_message, SET_VALUE=message, /APPEND
	WIDGET_CONTROL, state.heures, GET_VALUE = heures
	heurdeb = hms(heures(0))
	heurfin = hms(heures(1)) 
; Memorisation des parametres de la fenetre des isophotes
	winisoph = get_oldwindow(5,8)
	winprof = winisoph - 3
	isotrans = [!X.S, !Y.S]	; mise en memoire de la transf. DATA/DEVICE
; Lecture du fichier d'images, choix du parametre de Stokes a tracer 
	WIDGET_CONTROL, state.files(0), GET_VALUE = fichier
	lun=2
	err = newlec1d(lun,a,sa,FICHIER=fichier(0))
        WIDGET_CONTROL,state.bgroup,GET_VALUE=choix_qualifiers
        intens=Choix_Qualifiers(0)
        polar=Choix_Qualifiers(1)
;
; Cycle: Lecture d'un instant au curseur + trace du profil
; Le cycle est termine si le bouton central de la souris est pousse.
	win=get_window(winprof, winprof, TITLE='Profil')
	STARTSLID:
	wset, winisoph
	!X.S = isotrans(0:1) & !Y.S = isotrans(2:3)
	CURSOR,time,channel,/DATA,/CHANGE
	chtime = MSH(time)
		wset, winprof
		IF time GE heurdeb AND time LE heurfin THEN $
		prof1d,a,lun, HEURE = chtime, $
                   POLAR=polar,INTENSITE=intens,$
                   NOERASE=noerase
	IF !mouse.button NE 2 THEN GOTO, startslid	; (left button pressed)

	WSET, winisoph
	!X.S = isotrans(0:1) & !Y.S = isotrans(2:3)
ENDIF

; **** Selection d'une partie du champ avec le curseur (ZOOM) *****
IF ( ev.id EQ state.zoom ) THEN BEGIN
	message = "MOUSE:Left=Displace,Midle=Size,Right=Return"
	WIDGET_CONTROL, state.txt_message, SET_VALUE=message, /APPEND
	result=NRH1_getzone()
     IF N_ELEMENTS(result) EQ 1 THEN RETURN
     IF result(1) LT result(3) THEN BEGIN
        mainstruct.canaux(0)=result(1)
        mainstruct.canaux(1)=result(3)
     ENDIF ELSE BEGIN
        mainstruct.canaux(0)=result(3)
        mainstruct.canaux(1)=result(1)
     ENDELSE
     IF result(0) LT result(2) THEN BEGIN
        mainstruct.heures(0)=MSH(result(0))
        mainstruct.heures(1)=MSH(result(2))
     ENDIF ELSE BEGIN
        mainstruct.heures(0)=MSH(result(0))
        mainstruct.heures(1)=MSH(result(2))
     ENDELSE
     WIDGET_CONTROL, state.heures, SET_VALUE = mainstruct.heures

     WIDGET_CONTROL, state.files(0), GET_VALUE = fichier
	     err = newlitentete(1,fichier,file)
	     valres = MOTCLE(1, "LABEL")
	     IF valres EQ "EST-OUEST"  OR Valres EQ 'EAST-WEST' THEN $
		WIDGET_CONTROL, state.canauxew, SET_VALUE = mainstruct.canaux $
		ELSE $
		WIDGET_CONTROL, state.canauxsn, SET_VALUE =  mainstruct.canaux
ENDIF

IF ( ev.id EQ state.heures ) THEN BEGIN
	mainstruct.heures = ev.value
ENDIF

IF ( ev.id EQ state.canauxew OR ev.id EQ state.canauxsn ) THEN BEGIN
	mainstruct.canaux = ev.value
ENDIF


; on sauve les modifications
WIDGET_CONTROL, stash, SET_UVALUE = state
END


;**************************************************************************

PRO MENUTRACEISOPH
;          
;                             les variables globales (mainstruct)
;          Utilisations des fenetres 5,6,7,8 pour les affichages 
;          (mainstruct.windows(5:8))

common NRH1_HELIO,mainstruct

Title = "CONTOUR PLOT parameters"
;         initialisation du button group
column = 2  
nb=4          
Choix_Qualifiers = intarr(nb) ; pour les entrees du button group
FOR i = 0, nb-1 DO Choix_Qualifiers(i) = 0
Qualifiers = ['INTENS','POLAR','TEK','NOERASE']
Choix_Qualifiers(0)=1

loadct,12

files = lonarr ( 1 )

ISCROLL = 0
DEVICE, GET_SCREEN_SIZE = SCR_SIZE
RETBASE:			;Retour pour  Ascenseur
IF ISCROLL THEN $
    base = WIDGET_BASE (TITLE = title,/FRAME,/COLUMN, XOFFSET = 200,$
       YOFFSET = 200,/SCROLL,X_SCROLL_SIZE = Xss,Y_SCROLL_SIZE = Yss ) $
ELSE $
    base = WIDGET_BASE (TITLE = title,/FRAME,/COLUMN, XOFFSET = 200,$
                                                  YOFFSET = 200 )
base_file = WIDGET_BASE ( base,/COLUMN,/FRAME )
base_zone = WIDGET_BASE(base,/ROW,/FRAME)       

base_param = WIDGET_BASE (base,COLUMN=2,/FRAME )

base_zoom = WIDGET_BASE(base,/ROW,/FRAME)
zoom = WIDGET_BUTTON ( base_zoom, VALUE = 'ZOOM')
curseur = WIDGET_BUTTON ( base_zoom, VALUE = 'CURSOR')
proftemp = WIDGET_BUTTON ( base_zoom, VALUE = 'TIME PROF')
profcan = WIDGET_BUTTON ( base_zoom, VALUE = 'SCAN')
profcanslid = WIDGET_BUTTON ( base_zoom, VALUE = 'SLID. SCAN')
WIDGET_CONTROL,base_zoom,SENSITIVE = 0

base_qualifiers = WIDGET_BASE ( base,/ROW,/FRAME )
base_menu = WIDGET_BASE ( base_qualifiers )
base_menu2 = WIDGET_BASE ( base_qualifiers )

base_position = WIDGET_BASE (base,/COLUMN,/FRAME )
base_decomp = WIDGET_BASE(base, /COLUMN, /FRAME);	Recherche positions

fich=mainstruct.ewfile
files(0)= CW_pickfile_nrh(base_file, TITLE = " 1D images:", $
              VALUE = fich, FILTER = '*.[e,.n,.f]*')
Id_ewns = CW_BGROUP(base_file,['E-W','N-S'],$
	COLUMN=2, /EXCLUSIVE,SET_VALUE = Mainstruct.array) 
	ARRAY = 0

; Lecture de l'entete du fichier

err = newlitentete(1,fich,file)
IF err EQ 0 THEN BEGIN
   valmax=file.valmax
   canal= $
;     [fix(file.valorg(0)),fix(file.valorg(0)+file.paspix(0)*file.nbpix(0)-1)]
; pour passages des unites de canaux aux unites de rayons solaires
     [fix(file.valorg(0)),fix(file.valorg(0)+file.paspix(0)*file.nbpix(0))]
   heure=[string(MSH(file.heurdeb)),string(MSH(file.heurfin))]
   deltat = file.paspix(1)
ENDIF ELSE BEGIN
   valmax=0
   deltat = 0
   canal=[-512,512]
   heure=['00:00:00','00:00:00']
ENDELSE

; Parametrage du nombre de niveaux affiches
nlevels = WIDGET_SLIDER(base_param,MINIMUM = 1, MAXIMUM = 29,$
                        TITLE="Number of contours", VALUE = 12)
; Parametrage du pourcentage du niveau max affiche
niveau = WIDGET_SLIDER(base_param,TITLE="% of peak level",VALUE=50,$
                                  MINIMUM=1, XSIZE=300)

;Groupe de saisie des horaires de debut et de fin de visualisation
base_heures = WIDGET_BASE (base_zone, /COLUMN)
label_hr = WIDGET_LABEL(base_heures, VALUE = "Time:")
heures = CX_HEURES (base_heures,VALUE=heure)

base_canaux = WIDGET_BASE (base_zone, COLUMN=2)
label_ew = WIDGET_LABEL(base_canaux, VALUE = "EW channels:")
canauxew = CX_CANAUX (base_canaux,VALUE=canal)
label_sn = WIDGET_LABEL(base_canaux, VALUE = "SN channels:")
canauxsn = CX_CANAUX (base_canaux,VALUE=canal)

bgroup = CW_BGROUP(base_qualifiers,Qualifiers,COLUMN=column, $ 
                    SET_VALUE=Choix_Qualifiers,/NONEXCLUSIVE,/FRAME) 

; Echelle lin/log des isophotes
menu = WIDGET_LIST( base_qualifiers , $
	VALUE = ['linear1', 'linear2', 'logarithmic', 'HELP'] ) 
; Variable fonction = 0,1 (lin), 2 (log)
fonction = 0

; Affichage du maximum d'amplitude et de l'intervalle de temps
base_maxtim = WIDGET_BASE(base_qualifiers, /COLUMN)
maxvalue = CW_FIELD(base_maxtim, TITLE="Max. value : ",VALUE=valmax, $
                 /INTEGER,/RETURN_EVENT,XSIZE=8)
timestep = CW_FIELD(base_maxtim, TITLE = 'Delta t [ms] : ', VALUE=deltat, $
	/LONG, /RETURN_EVENT, XSIZE=8)


bgroup_control = CW_BGROUP ( base, ['UPDATE','QUIT','CONT','GRAY', $
	'GRAY+CONT', ' 3D '], /ROW, SPACE = 12 )


fichierpos =mainstruct.pewfile
fposition = CW_PICKFILE_NRH ( base_position,$
                            TITLE = 'Pos. file:',VALUE=fichierpos)
err = newlitentete(1,fichierpos,file)
IF err EQ 0 THEN Nbpos = File.nbpix(1)/File.facbloc(1) $
ELSE nbpos = 8

; *** Widget "Recherche de positions"

;bg_decomplab = WIDGET_LABEL(base_decomp, $
;	VALUE = 'Extract individual sources:')
rechpos = WIDGET_BUTTON ( base_decomp,VALUE = 'Extract individual sources')

base_decomp2 = WIDGET_BASE(base_decomp, COLUMN=3, /FRAME)

options =  ['Gaussian', 'Gauss + const', 'Gauss + linear', $
	'Gauss + parab.']
bg_decomp = WIDGET_LIST(base_decomp2,VALUE=options, $
	SCR_XSIZE=15*!d.x_ch_size, SCR_YSIZE=6*!d.y_ch_size)
txt_decomp = 0	; defaut: ajustement d'une Gaussienne
base_decomp21 = WIDGET_BASE(base_decomp2, COLUMN=3)
txt_decompcan = CW_FIELD(base_decomp21, Title = "Mean [chan]", /COLUMN, $
	/FLOAT, VALUE = 06.00, /RETURN_EVENT, XSIZE=5)
txt_decompsig = CW_FIELD(base_decomp21, Title = 'Weight', /COLUMN, /FLOAT, $
	VALUE = 0.50, /RETURN_EVENT, XSIZE=5)
txt_decompnbr = CW_FIELD(base_decomp21, Title = 'Iterations', /COLUMN, $
	/INTEGER, VALUE = 4, /RETURN_EVENT, XSIZE=5)
txt_decompeca =  CW_FIELD(base_decomp21, Title = 'Dist [chan]', /COLUMN, $
	/FLOAT, VALUE = 1.0, /RETURN_EVENT, XSIZE=5)
aidepos = WIDGET_BUTTON ( base_decomp21,VALUE = 'HELP')
;rechpos = WIDGET_BUTTON ( base_decomp21,VALUE = ' DO ')

WIDGET_CONTROL, base_decomp, SENSITIVE = 0

; *** ajout klk

tracepos = WIDGET_BUTTON ( base_position,VALUE = 'Overplot centroid position')

allpos = CW_SWITCH ( base_position, ['  ALL  ','SELECTION'], VALUE = 1)

numpos = CW_GRID ( base_position, indgen(nbpos), TITLE = 'positions')
WIDGET_CONTROL,numpos,SENSITIVE = 0
txt_message = WIDGET_TEXT( base, VALUE = ' ', YSIZE=3, /SCROLL)

state = { $
	base_zoom:base_zoom, $
          bgroup:bgroup ,nlevels:nlevels, bgroup_control:bgroup_control , $
	curseur:curseur, proftemp:proftemp, profcan:profcan, $
profcanslid:profcanslid, $
          files:files,zoom:zoom,$
          tracepos:tracepos, fposition:fposition, fonction:fonction, $
	base_decomp:base_decomp, bg_decomp:bg_decomp, $
	txt_decompcan:txt_decompcan, txt_decompsig:txt_decompsig, $
	txt_decompeca:txt_decompeca,txt_decompnbr:txt_decompnbr, $
	txt_message:txt_message, $
	txt_decomp:txt_decomp, $
	rechpos:rechpos, aidepos:aidepos, $
; *** ajout JB
	Id_ewns:Id_ewns,$	;pour les fichiers FITS
; ***
          numpos:numpos,allpos:allpos, base_position:base_position, $
          xrange:[0L,0L],yrange:[0L,0L],pastempo:0L,$
          debutemp:0L,menu:menu, maxvalue:maxvalue, niveau:niveau ,$
	timestep:timestep, $
          heures:heures,canauxew:canauxew, canauxsn:canauxsn }

WIDGET_CONTROL, WIDGET_INFO ( base, /CHILD ), SET_UVALUE = state 

WIDGET_CONTROL, base ,/REALIZE,TLB_GET_SIZE = Wid_size

	IF (Wid_size(0) GT 0.9*Scr_size(0) OR Wid_size(1) GT  0.9*Scr_size(1) $
			AND NOT Iscroll ) THEN BEGIN
		WIDGET_CONTROL, Base, /DESTROY
		Iscroll = 1
		XSS = Scr_size(0) * 0.5
		YSS = Scr_size(1) * 0.8
				GOTO, RETBASE
	ENDIF

XMANAGER,'MENUTRACEISOPH', base,GROUP_LEADER = mainstruct.root, /NO_BLOCK


END


 
