
;+ ***********************************************************************
; NAME:
;	CW_NRH_FILTER
;
; PURPOSE:
;	Ce compound widget definit un filter pour les fichiers FITS du NRH
;
; CATEGORY:
;	COMPOUND WIDGET
;
; CALLING SEQUENCE:
;	File = CW_NRH_FILTER()
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;	GROUP	ID du widget appelant
;	TYPE	Type de fichier selectionne
;				: 'i' images en coordonnees instrumentales
;				  'h' images en coordonnees heliographiques
;				  'a' harmoniques, Transformee de Fourier
;				  'f' flux
;				  's' centres actifs (sources)
;			par defaut TYPE = '?'
;
; OUTPUTS:
;	FILE	chemin et nom du fichier selectionne
;
; OPTIONAL OUTPUTS:
;	La structure contient au retour les parametres definis pour le
; filtrage des fichiers et le nom du fichier selectionne (avec le chemin)
;
; COMMON BLOCKS:
;	FICRH	structure contenant les parametres de filtre des fichiers
; et le nom du fichier selectionne (str_fic)
;			Description de la structure dans RH_2D.PRO

; PROCEDURE:
;	Les noms des fichiers traites correspondent a la definition suivante
;			----------------------
;		pour les fichiers de type nrh2, TYPE= 'H'
;			nrhR_ffff_cpi_yyyymmjj_hhmnss:ms_e.fts
;	nrh	nancay Radioheliographe
;	R	reseau:e: Est-ouest, n: Nord_sud, 2: 2 dimensions
;	ffff	frequence en 0.1Mhz
;	c	coordonnees: i instrumentales, h heliographiques, f flux
;			     s centres actifs,
;			     a harmoniques transformee de Fourier
;	pi	laisse libre a l'utilisateur, 
;			proposition:
;				p interpolation (0 a 9)ou autre information
;				  2**i points
;				i indice libre (0 a 9)
;	yyyymmjj	date
;	hhmmss		heure
;	ms		millisecondes
;	e	etat: b acquisition, c comprime, f decomprime (full)
;			-----------------------
;		ou bien pour les fichiers de la base, TYPE= 'B'
;			nfyymmjj.hhmnss.fts
;	f	frequence: a 164Mhz, b: 327Mhz

;
;	cette procedure utilise PICKFILE
; EXAMPLE:
;	FILE = CW_NRH_FILTER(GROUP= id)
;
; MODIFICATION HISTORY:
;	Ecrit par: J Bonmartin le 21/08/98 (bonmartin@obspm.fr)
;-*******************************************************************

PRO CW_NRH_FILTER_EVENT, event

COMMON FICRH, str_fic


COMMON CW_FILE_ID,str_id

DATE = str_fic.date
HR= str_fic.hr

ON_ERROR,2
parent = event.handler
stash = WIDGET_INFO(parent, /child)

       If event.id eq str_id.rd THEN   str_fic.rep = event.value

       If event.id eq str_id.rf THEN   str_fic.filtre = event.value

       If event.id eq str_id.freq THEN   str_fic.freq = event.value

       If event.id eq str_id.Year THEN  STRPUT,date , event.value, 0

       If event.id eq str_id.Month THEN  STRPUT,date, event.value,4

       If event.id eq str_id.Day THEN  STRPUT,date, event.value,6

       If event.id eq str_id.Hr THEN  STRPUT,hr , event.value, 0

       If event.id eq str_id.Min THEN  STRPUT,hr, event.value,2

       If event.id eq str_id.sec THEN  STRPUT,hr, event.value,4

	If event.id eq str_id.typ THEN STR_ID.vty = event.index

	If event.id eq str_id.tfic THEN BEGIN
			IF event.index EQ 0 THEN str_fic.typ = 'h'
			IF event.index EQ 1 THEN str_fic.typ = 'a'
			IF event.index EQ 2 THEN str_fic.typ = 'f'
			IF event.index EQ 3 THEN str_fic.typ = 'c'
	END

       If event.id eq str_id.rs THEN BEGIN
                     WIDGET_CONTROL, parent, /DESTROY
			Return
	end

       If event.id eq str_id.re THEN     Begin
                      WIDGET_CONTROL, parent, /DESTROY
                      Return
                  End

	str_fic.date= date
	str_fic.hr = hr
IF str_id.vty eq 0 THEN $
	str_fic.filtre= $
		'nrh2_'+str_fic.freq+'_'+str_fic.typ+'??_'+str_fic.date+'_'+ $
		str_fic.hr+':??_?.fts' $
ELSE BEGIN
	Freq = '?'
	If STRMID(str_fic.freq,0,3) EQ '164' THEN freq = 'a'
	If STRMID(str_fic.freq,0,3) EQ '327' THEN freq= 'b'
	str_fic.filtre = $
		'n'+freq+STRMID(str_fic.date,2,6)+'.'+str_fic.hr+'.fts'
	END

	WIDGET_CONTROL,str_id.rf,set_value=str_fic.filtre
end

FUNCTION CW_NRH_FILTER, GROUP = group, TYPE =type

COMMON CW_FILE_ID,str_id
COMMON FICRH, str_fic

 
STR_ID = {STR_ID,rd:0L, rf:0l, rs:0l, re:0L, freq:0L, Year:0L, Month:0L, $
		day:0L, hr:0L, Min:0L, Sec:0L, prt:0L, nom:0L, VTY:0, $
		Typ:0L, Tfic:0L}
STR_FIC.freq= '????'
STR_FIC.Date= '????????'
STR_FIC.Hr = '??????'
STR_FIC.Typ= '?'

if (NOT KEYWORD_SET(Group)) THEN group = 0

	Oldfil = Str_fic.filtre
		Str_fic.Filtre = 'nrh2*.fts'
If KEYWORD_SET (TYPE) THEN  Str_fic.filtre = $
		'nrh2_'+str_fic.freq+'_'+ type +'??_'+str_fic.date+'_'+ $
		str_fic.hr+':??_?.fts'

base = WIDGET_BASE( GROUP_LEADER= group, /column, /align_center, /MODAL)
	b= WIDGET_LABEL(base,value='SELECTION FILTER',/align_center)
 
str_id.rd = CW_FIELD(base, /string, Title = 'Directory:', /return_event, $
		      Xsize= 30,Value = str_fic.rep, Uvalue = 'Repert')
str_id.rf = CW_FIELD(base, /string, Title = 'Filter: ', /return_event, $
		     Xsize= 35, value = str_fic.filtre, Uvalue = 'Filtre')

base2 = WIDGET_BASE(base,column=3,/frame,/align_center)
		LIST= [' File NRH2','File NA/NB']
	str_id.Typ=WIDGET_DROPLIST(base2,value=list,Uvalue = 'Type')
		TFIC= ['Image','Harmon','Flux','Centers']
	str_id.Tfic=WIDGET_DROPLIST(base2,value=Tfic,Uvalue = 'Tfic')

	str_id.freq= CW_FIELD(base2, /return_event, Xsize=5, /STRING,$
				value = str_fic.freq,Title='Freq: ')
	str_id.Year= CW_FIELD(base2, /return_event, Xsize=5, $
			value = STRMID(str_fic.date,0,4),Title='Year :')
	str_id.Month= CW_FIELD(base2, /return_event, Xsize=5, $
			value = strmid(str_fic.Date,4,2),Title='Moun :')
	str_id.day= CW_FIELD(base2, /return_event, Xsize=5, $
			value = strmid(str_fic.Date,6,2),Title='Day  :')
	str_id.hr= CW_FIELD(base2, /return_event, Xsize=5, $
			value = strmid(str_fic.hr,0,2),Title='Hour :')
	str_id.Min= CW_FIELD(base2, /return_event, Xsize=5, $
			value = strmid(str_fic.hr,2,2),Title='Min  :')
	str_id.sec= CW_FIELD(base2, /return_event, Xsize=5, $
			value = strmid(str_fic.hr,4,2),Title='Sec  :')
str_id.nom= WIDGET_TEXT(base, Value=string(str_fic.file),Xsize=55)
str_id.rs= WIDGET_BUTTON( base, uvalue= 'Select', Value = 'Filter OK')
str_id.re= WIDGET_BUTTON( base, uvalue= 'Return', Value = 'Cancel')

WIDGET_CONTROL, base, EVENT_PRO= 'CW_File_event',/REALIZE

XMANAGER, 'CW_NRH_FILTER',base, /NO_BLOCK

return,str_fic.filtre

end

