
;+ ***********************************************************************
; NAME:
;	NRH_PICKFILE
;
; PURPOSE:
;	Cette procedure saist un  nom de fichier en fonction du contenu
;	de STR_FIC et initialise les structures correspondant au fichier
;	selectionne
;
; CATEGORY:
;	Fichier
;
; CALLING SEQUENCE:
;	NRH_PICKFILE, Str_fic, Str_inf, Str_lim, Physpar, Lfreq
;
; INPUTS:
;	STR_FIC	Structure de selection du fichier.
;
; KEYWORD PARAMETERS:
;	Nom
;
; OUTPUTS:
;	STR_INF	Structure d'information du fichier (NRH_STR_INFIC)
;		Cette structure doit etre cree avant l'appel
;	STR_LIM	Structure de limite des traitement (NRH_STR_LIM)
;		Cette structure doit etre cree avant l'appel
;	PHYSPAR	Tableau de chaines des parametres physiques existant 
;		dans le fichier
;	LFREQ	Tableau de chaines des frequences ou longueurs d'ondes 
;		existant dans le fichier
;
; COMMON BLOCKS:
;	Non
;
; SIDE EFFECTS:
;	Le contenu de la structure STR_LIM est reinitialisee avec les
;	valeursa contenues dans le fichier 
;
; RESTRICTIONS:
;	Les structures STR_INF et STR_LIM doivent exister avant l'appel
;
; PROCEDURE:
;	Cette procedure appelle:
;		DIALOG_PICKFILE pour saisit le nom du fichier
;		NRH_FOPEN pour effectuer les initialisations en fonction 
;				du fichier
;	Avant l'execution l'appel a CW_NRH_FILTER permet de fixer le chemin
;	et le filtre de selection du fichier
;
; EXAMPLE:
;	Please provide a simple example here
;
; MODIFICATION HISTORY:
;	Ecrit par:J Bonmartin (obspm.fr) le 24/08/98
;-*******************************************************************


PRO NRH_PICKFILE, Str_fic, Str_inf2d, Str_lim, Physpar, Lfreq 

		  file = $ 
             dialog_pickfile( PATH= str_fic.rep, FILTER= str_fic.filtre, $
			/Read, /Noconfirm)

;**********Pour se proteger d'une mauvaise selection de fichier
		SZ = SIZE (FINDFILE( File))
		IF SZ(1) NE 1 THEN BEGIN	;fichier non unique
			PHYSPAR = ' ' & LFREQ = ' ' 
			RETURN & END
;*************************************************************

                IF file NE '' THEN $
                       NRH_FOPEN, File, Str_inf2d, $
                               Str_lim, Physpar, Lfreq 
		If File NE '' THEN $
                       Str_fic.file = file

END