
;+ ***********************************************************************
; NAME:
;	READ_NRHIM
;
; PURPOSE:
;	Cette procedure lit une image dans un fichier FITS
;
; CATEGORY:
;	Fichier
;
; CALLING SEQUENCE:
;	READ_NRHIM, File, Heure, Str_inf, Donnee, Str_im
;
; INPUTS:
;	FILE	Nom du fichier
;
;	HEURE	Chaine contenant l'heure (hh:mn:ss:ccc), l'heure est prise
;		en compte si le keyword INDIC n'est pas present et si le
;		fichier contient plusieurs images
;
;	STR_INF	Structure d'information du fichier resultant NRH_FOPEN
;	
; KEYWORD PARAMETERS:
;	STOKES	Parametre de Stokes pour NRH: STOKESV si =1, sinon
;			STOKESI
;	INDIC	Indice de l'image dans le fichier
;
; OUTPUTS:
;	DATA	Tableau contenant l'image
;
;	STR_IM	Structure d'information liee a l'image (NRH_STR_IMAGE)
;
; COMMON BLOCKS:
;	Non.
;
; PROCEDURE:
;	Cette procedure utile STR_INF.INSTRUME pour selectionner
;	la procedure de lecture associee a l'instrument
;	Instuments reconnus:
;		NRH
;		EIT SOHO
;		YOHKOH
;		PNG
;
; EXAMPLE:
;	Please provide a simple example here
;
; MODIFICATION HISTORY:
;	Ecrit par:J Bonmartin (obspm.fr) le 24/08/98
;-*******************************************************************

PRO READ_NRHIM, File, Heure, Str_inf2d, Donnee, Str_im, $
				STOKES = Stokes,INDIC= indic


		IF Str_inf2d.Content EQ '2DI' THEN BEGIN		; NRH2 et HSSI
				RD_NRH2I, File, Heure, Str_inf2d, $
							DONNEE, Str_im,$
				STOKES = Stokes, INDIC= indic 
			END

		IF Str_inf2d.Content EQ '2DB' THEN $
			RD_STDFITS, File, Str_inf2d, Donnee, Str_im $
		ELSE IF Str_inf2d.Content EQ 'STD' THEN $
			RD_STDFITS, File, Str_inf2d, Donnee, Str_im $
		ELSE IF Str_inf2d.instrume EQ 'YOHKOH' THEN $
			RD_STDFITS, File, Str_inf2d, Donnee, Str_im $
		ELSE IF Str_inf2d.instrume EQ 'PNG' THEN $
			RD_NRHPNG, File, Str_inf2d, Donnee, Str_im  $
		ELSE IF Str_inf2d.instrume EQ 'JPG' THEN $
			RD_NRHJPG, File, Str_inf2d, Donnee, Str_im  $
		ELSE IF Str_inf2d.instrume EQ 'GIF' THEN $
			RD_NRHGIF, File, Str_inf2d, Donnee, Str_im

; Entrer les autres instruments, (faire suivre l'ordre precedent de $)


END
