
;+ ***********************************************************************
; NAME:
;	NRH2_TRISOPH
;
; PURPOSE:
;	Cette procedure trace les isophotes de la projection 1D du nrh2
;	sur le modele du trace de NRH1
;
; CATEGORY:
;	NRH2 project
;
; CALLING SEQUENCE:
;	 NRH2_TRISOPH, Data, Time, Lim, Info,
;
; INPUTS:
;	DATA	Tableau resultant du calcul de projection
;	TIME	Tableau des temps
;	LIM	Structure  NRH_STR_LIM
;	INFO	Structure NRH_STR_INFIC
;	
; KEYWORD PARAMETERS:
;	NLEVELS	Nombre de niveaux de contours (10 par defaut)
;	MAXP	Niveau maximum en % du max de Data 
;	MINP	Niveau minimum en % du max de Data
;	LAB_TITLE	StructureNRH_STR_LABEL
;
; OUTPUTS:
;	Traces d'isophotes
;
; COMMON BLOCKS:
;	Non
;
; EXAMPLE:
;	Please provide a simple example here
;
; MODIFICATION HISTORY:
;	Ecrit par: J Bonmartin (obspm.fr) le 09/07/99
;-*******************************************************************

PRO NRH2_TRISOPH, Data, Time, Lim, Info, $
	NLEVELS=Nlevels, MAXP=Maxp, MINP=Minp, $
	LAB_TITLE=Str_Label

IF NOT KEYWORD_SET (NLEVELS) THEN NLEVELS=10
IF NOT KEYWORD_SET (MAXP) THEN MAXP=90
IF NOT KEYWORD_SET (MINP) THEN MINP=5

	Rd= IND_RS(Lim.Xd, Info.Xcen, Info.ray)
	Rf= IND_RS(Lim.Xf, Info.Ycen, Info.ray)
	SX= Size(Data)
	Incre=(Rf-rd) / Sx(2)
	YAXIS = Rd + Findgen(Sx(2)) * Incre 

	ST = Size(Time)
	XAXIS = LONARR(St(1))
	FOR I = 0, St(1)-1 DO Xaxis(i) = Hms(Time(i))
	Taxis= [Xaxis[0], Xaxis[St(1)-1]]

;			**** Label Titre
	LABEL= ''
IF N_TAGS(Str_label)NE 0 THEN BEGIN 
	LFREQ = ''
	LDATE= ''
	LHR= ''
	LINSTR= ''
	LPHYS= ''

	IF Str_label.Date EQ 1 THEN Ldate = Info.Date

	IF Str_label.freq EQ 1 THEN Lfreq = STRING(Info.Freq,FORMAT='(F5.1)')

	IF Str_label.heure EQ 1 THEN Lhr = STRING(time(0))+ ' to '+$
				STRING(Time(st(1)-1))

	IF Str_label.instrume EQ 1 THEN Linstr = Info.Instrume

	IF Str_label.Physpar EQ 1 THEN BEGIN
		IF Info.Physpar EQ 1 THEN Lphys = 'I' $
			ELSE Lphys = 'V'
	ENDIF

	Label = str_label.free+' '+Linstr+' '+Lphys+' '+$
			lfreq+' '+ldate+' '+lhr

ENDIF 
;			******FIN Label
	MAX = MAX(Data)
	Max = Max*Maxp/100 & Min = Max*Minp/100 

	MILSEC_AXIS, Taxis, tickv, tickl, ticks

	CONTOUR, Data, Xaxis, Yaxis,$
		NLEVELS= Nlevels,MAX_VALUE=Max, MIN_VALUE=Min, $
		XSTYLE=1, YSTYLE=1,YRANGE=[Rd,Rf], $
		XRANGE=[Taxis],XTICKV=Tickv,XTICKS=Ticks[0],XMINOR=ticks[1],$
		XTICKNAME=Tickl, TICKLEN = -0.02, TITLE=Label, $
		CHARSIZE=0.6, YMARGIN = [8,8]


END