
;+ ***********************************************************************
; NAME:
;	NRH_ROTIM
;
; PURPOSE:
;	Cette fonction effectue la rotation d'une image
;
; CATEGORY:
;	NRH2 project
;
; CALLING SEQUENCE:
;	Image= (NRH_ROTIM, Data, Angle, [Xcim], [Ycim]) XCEN=Xcen, YCEN=Ycen
;
; INPUTS:
;	DATA	Tableau d'origine
;	ANGLE	Angle de rotation en degres
;	
; KEYWORD PARAMETERS:
;	XCEN, YCEN	Coordonnees de l'image contenue dans DATA
;
; OUTPUTS:
;	Image apres rotation
;
; OPTIONAL OUTPUTS:
;	XCIM, YCIM	coordonnees du centre de l'image apres rotation si
;			XCEN et YCEN ont ete declares
;
; COMMON BLOCKS:
;	Non
;
; EXAMPLE:
;	Please provide a simple example here
;
; MODIFICATION HISTORY:
;	Ecrit par: J Bonmartin (obspm.fr) le 09/07/99
;-*******************************************************************

FUNCTION NRH_ROTIM, Data, Angle, Xcim, Ycim, XCEN=Xcen, YCEN=Ycen


	SZ= SIZE(Data) & DIM = FIX(SQRT(SZ(1)^2 + SZ(2)^2))
		IMAGE= FLTARR (Dim, Dim)

IF NOT KEYWORD_SET(XCEN) THEN Xcen=Sz(1)/2
IF NOT KEYWORD_SET(YCEN) THEN Ycen=Sz(2)/2

	Xl = Dim/2 - Sz[1]/2
	Yl = Dim/2 - Sz[2]/2
	Image[Xl:Xl+Sz[1]-1,Yl:Yl+Sz[2]-1] = Data
	IMAGE = ROT(Image, Angle)

	XCIM=Xcen + Xl
	YCIM=Ycen + Yl

RETURN, Image
END

