
;+ ***********************************************************************
; NAME:
;	NRH_LEVEL_LIN
;
; PURPOSE:
;	Cette fonction calcule une table de niveaux pour les traces de
;	contours
;
; CATEGORY:
;	Visualisation d'images
;
; CALLING SEQUENCE:
;	LEVEL= NRH_LEVEL_LIN( IMAGE, NLEVEL, MAXC, MINC )
;
; INPUTS:
;	IMAGE	Image sur laquelle sont calcules les niveaux
;
;	NLEVEL	Nombre de niveaux
;
;	MAXC	Pourcentage du maximum de l'image pris en compte pour le
;		niveau maximum
;
;	MINC	Pourcentage du maximum de l'image pris en compte pour le
;		niveau minimum
;	
; KEYWORD PARAMETERS:
;
;
; OUTPUTS:
;	LEVEL	Tableau contenant les niveaux
;
; COMMON BLOCKS:
;	
; PROCEDURE:
;	Cette procedure calcule le nombre de niveaux correspondants entre
;	le max et le min
;	Le tableau LEVEL est retourne avec les valeurs negatives et positives
;	des niveaux afin de pouvoir tracer les niveaux negatifs si l'image 
;	en contient.
;	Les niveaux negatifs peuven etre traces avec un symbole different
;	dans CONTOUR en gerant C_LINESTYLE
;
; EXAMPLE:
;	Please provide a simple example here
;
; MODIFICATION HISTORY:
;	Ecrit par:J Bonmartin le 10/08/98 (obspm.fr)
;-*******************************************************************

FUNCTION NRH_LEVEL_LIN, IMAGE, NLEVEL, MAXC, MINC

	LEVEL = FLTARR (Nlevel * 2)

	MAXIM = FLOAT(MAX(Image, MIN = Minv))
		IF ABS(Minv) GT Maxim THEN Maxim = ABS(Minv)
	PAS = Maxim*(MAXC - MINC) / (Nlevel-1)/100
	MINIV = Maxim * Minc /100
	MINLEV = -Maxim* (Maxc-Minc) /100 -Miniv

	FOR I = 0, Nlevel-1 DO BEGIN
		LEVEL(I) = Minlev + (Pas * I)
		LEVEL( I+Nlevel) = Miniv +(Pas * I)
	END


RETURN, LEVEL

END