;+ ***********************************************************************
; NAME: PUT_STR_DP
;	
; PURPOSE:
;       Avant l'appel a rh_dpatchfits_nrh, on regroupe dans
;       une structure tous les arguments d'appel.
;
; CATEGORY:
;	Traitement de fichiers NRH
;
; CALLING SEQUENCE:
;	
; INPUTS:  
;   tous les parametres pour rh_dpatchfits_nrh
;
; OUTPUTS:
;   str_dp  : structure contenant ces parametres
;
; MODIFICATION HISTORY:
; 01 oct    : ecrit par: A. Bouteille
; 02 mar  5 : - ajout de sel_ant dans la liste d'entree pour eliminer les 
;		  antennes a problemes.
; 02 mar  7 : - ajout de crit_arret, larg_2 devient un fltarr(2).
;	 27 : - ajout de rot_p .
; 03 jun 24 : - ajout des parametres de recalibration
;	      - ----------------------- complement du pave central (recentrage)
; 03 sep 17 : - ajout de : 
;		. i_corr_pos_ant (correction de position des antennes)
;		. i_source_cal, h_cal
; 04 sep 22 : - chgt de la recalibration manuelle : disparition de g_E2 etc,
;		  introduction de phi_ew_ns, g_flag, g_amp, g_phi .
; 05 fev  5 : - ajout de t_integ (temps d'integration prealable en sec) a la 
;		  liste d'entree.
; 05 fev  9 : - ajout de  phi_w_e  a la liste d'entree.
; 06 mar  3 : - ajout de une dimension sipplementaire (2) a phi_ew_ns, phi_w_e,
;		   g_amp, g_phi pour pouvoir corriger manuellement et simulta-
;		   nement non polar et polar. En fait la seule modif dans ce 
;		   programme est ce commentaire.
;	      - ajout de  i_affich_lobe  et  i_mul_pol  a la liste d'entree.
;
;-*******************************************************************


PRO PUT_STR_DP, $
;  Entree : 
        version, i_simulation,                                       $ 
;     Temps d'integration prealable
	t_integ,						     $
;     Caracterisation de l'image
	n_period,  mode_interpol,  rot_p,  np,  larg,  t_b,	     $
	i_affich_lobe, i_mul_pol, 				     $
;     Suppression (eventuelle) des harmoniques de certaines antennes
	ie0_ew,    ie0_ns,    ie1_ew,    ie2_ew,   ie2_ns,	     $
	ins45_ew,  ins45_ns,  i_redond,  gap_ns0,  sel_ant, 	     $
;     Cas particulier des sources de controle de calibration
	i_con_cal,					  	     $
;     Correction de position d'antennes et recalibration
	i_corr_pos_ant,  i_recalib,  i_source_cal,   h_cal,  ch_cor, $
	phi_ew_ns,  phi_w_e,  i_corr_g,  g_flag,  g_amp,  g_phi,     $
;     Complement du pave central (harm multiples impairs de 50m)
	i_pave_plein,	i_recentrage,  x_centrage,  y_centrage,      $
;     Calibration et clean
	mode_clean,  x_dom,  y_dom,  dr_dom,			     $
	i_ech_min,  i_ech_max ,  a_in,  a_ex,			     $
	a_red_fil,  itermax,  gain_iter,  jmax,  crit_arret,	     $
	i_seuil,  i_test_arret,  i_stop_image,  i_ecrete,  frac_ecr, $
	tete_fich,						     $
	fac_crois_res,	coeff_seuil, frac_flux,  		     $
	i_lissage,  poids_c, larg_2,				     $
;  Sortie :
	str_dp


;  Initialisation de la structure a ses valeurs par defaut :
        DEF_STR_DP, str_dp

	str_dp.version		= version
	str_dp.i_simulation	= i_simulation
        str_dp.t_integ		= t_integ
        str_dp.n_period		= n_period
        str_dp.mode_interpol	= mode_interpol
	str_dp.rot_p		= rot_p
        str_dp.np		= np
        str_dp.larg		= larg
        str_dp.t_b		= t_b
        str_dp.i_affich_lobe	= i_affich_lobe
        str_dp.i_mul_pol	= i_mul_pol

        str_dp.ie0_ew		= ie0_ew
        str_dp.ie0_ns		= ie0_ns
        str_dp.ie1_ew		= ie1_ew
        str_dp.ie2_ew		= ie2_ew 
        str_dp.ie2_ns		= ie2_ns
        str_dp.ins45_ew		= ins45_ew 
        str_dp.ins45_ns		= ins45_ns
        str_dp.i_redond		= i_redond
        str_dp.gap_ns0		= gap_ns0
	str_dp.sel_ant		= sel_ant

	str_dp.i_con_cal        = i_con_cal
 
	str_dp.i_corr_pos_ant	= i_corr_pos_ant
        str_dp.i_recalib	= i_recalib
        str_dp.i_source_cal	= i_source_cal
        str_dp.h_cal		= h_cal
	str_dp.ch_cor           = ch_cor  

	str_dp.phi_ew_ns        = phi_ew_ns
	str_dp.phi_w_e          = phi_w_e
	str_dp.i_corr_g         = i_corr_g 
	str_dp.g_flag           = g_flag
	str_dp.g_amp            = g_amp
	str_dp.g_phi            = g_phi
  
        str_dp.i_pave_plein	= i_pave_plein
	str_dp.i_recentrage	= i_recentrage
	str_dp.x_centrage	= x_centrage
	str_dp.y_centrage	= y_centrage

        str_dp.mode_clean	= mode_clean 
        str_dp.x_dom		= x_dom 
        str_dp.y_dom		= y_dom 
        str_dp.dr_dom		= dr_dom 
        str_dp.i_ech_min	= i_ech_min
        str_dp.i_ech_max	= i_ech_max
        str_dp.a_in		= a_in
        str_dp.a_ex		= a_ex
        str_dp.a_red_fil	= a_red_fil
        str_dp.itermax		= itermax
        str_dp.gain_iter	= gain_iter
        str_dp.jmax		= jmax
        str_dp.crit_arret	= crit_arret
        str_dp.i_seuil		= i_seuil
        str_dp.i_test_arret	= i_test_arret
        str_dp.i_stop_image	= i_stop_image
        str_dp.i_ecrete		= i_ecrete
        str_dp.frac_ecr		= frac_ecr
        str_dp.tete_fich	= tete_fich
        str_dp.fac_crois_res	= fac_crois_res
        str_dp.coeff_seuil0	= coeff_seuil(0, *)
        str_dp.coeff_seuil1	= coeff_seuil(1, *)
        str_dp.coeff_seuil2	= coeff_seuil(2, *)
        str_dp.frac_flux	= frac_flux
        str_dp.i_lissage	= i_lissage
        str_dp.poids_c		= poids_c
        str_dp.larg_2		= larg_2

    end
