
PRO ECRETE, im, frac			; im en entree et en sortie.

; But : ecreter une image pour tracer. Usage typique : voir des details au pied
;	d'un centre intense. Quand frac est petit (<0.1 par ex) l'ecretage
;	peut concerner des parties de signe differents de l'image, et l'ecre-
;	tage doit evidemment conserver le signe.

; Notations :
; im	image en entree et sortie.
; frac	fraction du max en dessous duquel on ecrete

    max_abs = max(abs(im), ip)
    dom_p = where((abs(im) ge frac*max_abs) and (im gt 0), cp)	; ecretage >0
    dom_n = where((abs(im) ge frac*max_abs) and (im lt 0), cn)	; ecretage <0
    
    if (cp gt 0) then im(dom_p) =   frac  * max_abs
    if (cn gt 0) then im(dom_n) = - frac  * max_abs
	; si frac n'est pas tout petit il peut arriver que l'un des domaines
	;   soit vide.  

    end			; fin de ECRETE.

;______________________________________________________________________________

; appele par DPATCHFITS.PRO : 				68 PARAMETRES !!
;   . une fois pour l'image non polar,
;   . --------------------- polar.

PRO RH_CALC_IMAGE_HELIO, $
    ; Entree
	; Caracterisation de l'image :
	    jul_dat,   correl,    freq,    n_image,  npi,      n_bord,	$
	    i_polar,   ch_polar,  anorm_1_s,  anorm_1_p,		$
	    n_period,  mode_interpol,  np,  champ_helio,  t_b, 		$
	    i_affich_lobe,						$
	; Suppression (eventuelle) des harmoniques de certaines antennes
	    ie0_ew,   ie0_ns,   ie1_ew,   ie2_ew,   ie2_ns, 		$
	    ins45_ew,  ins45_ns,  i_redond,  gap_ns0,  sel_ant,		$
            iaa_panne = iaa_panne,                                      $
	; Complement du pave central (harm multiples impairs de 50m)
	    i_pave_plein, i_recentrage,	x_centrage,  y_centrage,	$
	; Donnees pour une image :
	    i_simulation,  tf_objet_simule,  objet_simule,		$
	    harm_N,  rg,  sg,  domega_i,  champ,			$
	; Clean :
	    mode_clean,   dom_c,        dom_a_i,    dom_a_h,		$
	    i_ech_min,    i_ech_max ,   a_in,       a_ex,		$
	    a_red_fil,    itermax,      gain_iter,  jmax,  crit_arret,	$
	    i_seuil,      coeff_seuil,  frac_flux,  fac_crois_res,	$
	    i_test_arret, i_stop_image, i_ecrete,   frac_ecr,		$
	    tete_fich,							$
	    i_lissage,   poids_c,  larg_2,				$
	; Divers
	    i_sys_lin,							$
    ; Sortie :
	    x_centrage_np,   y_centrage_np,				$
	    l_th,  im_hel_p, flux_int_s, flux_compact

		;  im_hel_p  image heliographique propre et normalisee.

; Commentaires sur le calcul du flux et l'aliasing apres la normalisation, vers
;	la ligne 700
;
; Fonction :
;   - suppression eventuelle de certains harmoniques,
;   - controle de la distribution des harmoniques dans "harm_N", 
;   - calcul de l'image (non polar ou polar) "sale" sur la grille des coord 
;	interferometriques entieres et normalisation,
;   - clean eventuel (facon JM ou CM) et autocalibration eventuelle facon JM,
;   - periodisation de l'image interferometrique
;   - calcul de l'image heliographique finale (soleil rond) par:
;	. interpolation de la precedente,
;	. normalisation:
;	    . si l'image est en brillance, on la normalise de sorte que son 
;		flux soit simplement la somme de ses points (npi*npi pour 
;		l'image interferometrique, np*np pour l'image heliographique).
;	    . si l'image est en temp. de brillance, elle est en Kelvins.
;   - calcul des flux. La reconstitution de la composante continue (non acqui-
;		se) de l'image est faite dans MALC_IM_2D, qui est appele par
;		CALC_IM_HELIO.
;   - presentation des images sales et propres (si i_stop_image=1).


; Modifications :
; 00 nov 30: Ajout de dom_c a la liste des arguments pour transmission a 
;		RH_CLEAN (a cette epoque REGCLEAN). C'est le tableau des indi-
;		ces de l'image definissant le domaine de l'image interferome-
;		trique auquel on limitera la recherche des composantes clean.
;	     Deplacement de l'appel a RH_CLEAN (a cette epoque REGCLEAN) : ini-
;		tialement en fin de MALC_IM_2D, il est deplace dans CALC_IMAGE_
;		HELIO juste apres l'appel a MALC_IM_2D.
;    dec  6: Periodisation de l'image interferometrique (etude des CMEs etc.)
;        11: Normalisation de l_th a un maximum de 1, adapte a son usage dans
;		P_CLEAN_2D. Modif faite simultanement avec la modif de normali-
;		sation de i_fil dans RH_CLEAN (a cette epoque WCLEAN).
; 01 jan  4: visualisation du contour du domaine dom_c sur l'image interfero-
;		metriquedans WCLEAN (trace du contour dom_a_i).
;        24: mode_clean passe en argument.
;    fev 12: passage des arguments pour le clean definis dans dpnew.
; 01 jul  3: passage de i_sys_lin a MALC_IM_2D
;	  4: changement de tb (0 ou 1) en t_b (1 ou 2).
;         6: passage de npi et np en entree (au lieu de l'ancien npew seul).
;        19: le choix du mode d'interpolation pour passer de l'image interfe-
;		rometrique a l'image heliographique est remont'e dans DPNEW,
;		compte tenu que les fft sont lentes.
;    sep 21: ajout de "objet_simule" dans la liste d'appel pour transmission
;		a RH_CLEAN (a cette epoque REGCLEAN).
;    oct 18: insertion de RH_NORM_IM en tete du fichier.
; 02 jan 25: . introduction de i_redond 
;	     . modif du systeme de mise a zero des harm de
;	         certaines antennes, pour ne pas faire la moyenne avec zero en 
;	         cas de redondance.
; 02 mar  5: ajout de sel_ant dans la liste d'entree pour eliminer les antennes
;		 a aproblemes
; 02 jun  7: ajout de objet_simule_lisse a la liste de sortir de RH_CLEAN pour
;		comparaison dans  RH_CALC_IMAGE_HELIO.
; 03 mar 13: ajout de  i_recentrage  a la liste d'entree pour passage a 
;		RH_MALC_IM_2D .
; 03 mar 19: ajout de  x_centrage  et  y_centrage  a la liste d'entree pour 
;		passage a RH_MALC_IM_2D.
; 03 mar 20: ajout de  tf_objet_simule  (liste d'entree de RH_CALC_IMAGE_HELIO)
;	     ajout de  tf_im_int_s  et  tf_l_th  a la liste d'entree de 
;								RH_CLEAN.
; 04 jan 19: adaptation aux antennes AA
; 05 dec  2: amelioration de la determination de la base de l'image inteferome-
;	       trique dans RH_MALC_IM_2D (commentaires dans RH_MALC_IM_2D).
; 06 fev 27: incrustation de la section a mi-puissance du lobe synthetique 
;	       heliographique dans dans les images heliographiques propres :
;	       . 512 * 512 pour controle dans RH_CALC_IMAGE_HELIO
;	       . np  * np  passees a RH_DPATCHFITS_NRH  pour ecriture fits.
; 06 mar  6: introduction de  i_affich_lobe  dans la liste d'entree.
; 06 mar  8: ajout de  i_polar  (en plus de  ch_polar  deja present) dans la
;		liste d'entree pour reconnaitre le cas ou on ne calcule pas 
;		l'image non polar. Dans ce cas on calcule le facteur de norma-
;		lisation de l'image polar helio a partir des donnees polar 
;		seules (et non plus a partir de l'image non polar), mais le 
;		resultat est alors plus plus incertain.
;	     ajout de  anorm_1_s  et anorm_1_p  a la liste de sortie.
; 06 mar 20: correction de la gestion du calcul des images et lobes heliogra-
;		phiques selon que mode_clean est nul ou non. Idem pour le cal-
;		cul de la section du lobe synthetique.
; 06 mar 22: correction du calcul des lobes sales et propres helio quand l'ima-
;		ge interferometrique est etendue a plus d'une periode interfe-
;		rometrique
;	     gestion du cas des scrutation identiquement nulles : saut de tout
;		et production de sorties nulles.
; 05 jul 2011: Gestion de la panne du correlateur depuis le
;             27/11/2009 (antennes AA3 et AA4 avec NS12 a NS16)

; Notations:
; champ	    drapeau (calcule par RH_GRILLE) mettant a zero l'image heliogra-
;	      phique la ou rg et sg correspondent a des points exterieurs au 
;	      champ interferometrique periodise. Cela fait strictement double 
;	      emploi avec "miss=0" dans le INTERPOLATE dans RH_CALC_IMAGE_HELIO
;	      "champ" est transmis par RH_DPATCHFITS_NRH a :
;		. RH_CALC_IMAGE_HELIO, ou il est donc de peu d'utilite,
;		. P_CLEAN_2D, ou il ne sert aussi qu'a des delouiseries dans
;		    un affreux mic-mac ou on l'appliquait aux pts de l'image
;		    interfero alors qu'il concerne les pts de l'image helio,
;		    et au calcul iteratif purement delouisien dans le cas (non
;		    revu) ou on fixe d'avance le nbre d'iterations. 	      
; correl    tableau (2, nbre_harm) indiquant les numeros des 2 antennes corres-
;	      pondant a chacun des  nbre_harm  harmoniques. Permet dans 
;	      RH_MALC_IM_2D le passage de harm_N au tableau "harm" des harmo-
;	      niques a 2 dimensions.
; dom_c	    domaine de l'image interferometrique auquel on limite la recherche
;	      des composantes clean.   dom_c correspond a un disque sur l'image
;	      heliographique, de rayon  dr_dom .  ***
; dom_a_i   domaine annulaire etroit entourant dom_c sur l'image interfero.
; dom_a_h   ---------------------------------------------------- helio
; flux_int_s  est le flux interferometrique "sale" en sfu. C'est la somme des 
;		points de l'image interferometrique sale obtenue par fft dans 
;		RH_MALC_IM_2D, ou plutot la somme des points ou l'image est 
;		notable (commentaires du 2 dec 2005 dans RH_MALC_IM_2D). La
;	        transformation en image heliographique est faite ensuite. 
;	      Si t_b=1, l'image helio est normalisee de facon que la somme de
;	        ses points donne flux_int_s.
;	      Si t_b=2, l'image helio est normalisee de facon qu'elle soit en
;		Kelvins.
; harm_N    vecteur des nbre_harm (576 ou 648 avec les AA) correlations. Rempli
;	      en non-polar ou polar juste avant l'appel a RH_CALC_IMAGE_HELIO.
;
; mode_clean	choix du mode de clean :
;		  . 0 pas de clean 
;		  . 1 pour clean normal sur champ entier,
;		  . 2 pour clean normal sur champ entier mais on n'ajoute pas 
;		       le residu a l'image synthetique.
;		  . 3 pour chercher partout les comp clean mais ne pas resti-
;			tuer celles interieures au domaine dom_c defini par 
;			x_dom, y_dom et dr_dom.
;		  . 4 pour retirer les composantes clean de dom_c et obtenir 
;			une image residuelle non nettoyee du reste.
;		  . 5 pour nettoyer sur dom_c seulement.
;		  Voir commentaires plus complets dans RH_DPNEW.
; npi	    dimension de l'image interferometrique, figee a 128 apres le 6 jul
;	      01.
; np	    dimension demandee pour l'image heliographique finale (soleil rond
;	      dans un champ carre de largeur totale "champ_helio" en RS). 
;	      L'image finale doit etre suffisamment echantillonnee pour etre 
;	      interpolable. Si "champ_helio" est > champ interferometrique 
;	      avec E0 il faut donc prendre npew > 128 .
; n_period    nb de periodes a reproduire sur chaque axe avant interpolation.
; rg, sg    coord interferometriques des points ou l'image heliographique est 
;	      calculee. Leur origine est au coin inferieur gauche de l'image
;	      interferometrique, eventuellement periodisee. Ils sont ainsi cal-
;	      cules dans GRILLE.
; tf_objet_simule   est centree en milieu de tableau.

    version_solarsoft = 1       ; 0 version test de Claude 1 SolarSoft
                                ; permet de gerer impressions et stop
 
    nbre_harm = n_elements(harm_N)
    if (nbre_harm eq 576) then nbre_ant = 44
    if (nbre_harm gt 576) then nbre_ant = 48	; et nbre_harm = 648 en atten-
						;   dant mieux.

; Detection de scrutations pathologiques identiquement nulles
    dom_non_nul = where(abs(harm_N)  ne  0, ic_non_nul)
    if (ic_non_nul  lt  5) then begin
	    ; en principe ca devrait etre  if (ic_non_nul  eq  0), mais on
	    ;   elimime aussi des scrutations avec un nombre anormalement bas
	    ;   d'harmoniques non nuls.
	print, ''
	print, 'RH_CALC_IMAGE_HELIO : scrutation identiquement nulle'
	ch_format = "(4x, 'heure : ', i2.2, ':', i2.2, ':', i2.2, " + $
		"':', i2.2, 4x, i4, ' MHz,  ')"
	print, format=ch_format, hmsc, freq
	print, ''
	x_centrage_np = 0		& y_centrage_np = 0
	l_th     = fltarr (npi, npi)
	im_hel_p = fltarr (np , np)
	flux_int_s = 0			& flux_compact = 0
	print, 'Taper .c  pour continuer.  Toutes les sorties de ' + $
	    'RH_CALC_IMAGE_HELIO seront'
	print, '             nulles.'
;	stop
	goto, a2		; vers fin de RH_CALC_IMAGE_HELIO
    endif

; SUPPRESSION EVENTUELLE DES HARM OBTENUS AVEC :
	; . E0 (periode double en EW et distrib asym dans le plan (u,v)),
	; . E1 avec les EW,
	; . E2 (distrib asym dans le plan (u,v), 
	; . NS45 (barre en u pour v=45)
	; . harmoniques obtenues avec des antennes trop proches de NS0.
	; . suppression des doublures mediocres (bases NS courtes avec NS0 qui
	;     est parasitee, de toutes les bases EW avec NS0).
	; . 
	; Voir description rangement dans harm_N dans RH_MALC_IM_2D.
	; Rem (25 jan 02) : la mise a zero des harmoniques resoud le probleme 
	;   de leur suppression :
	;   . s'il n'y a pas redondance, c'est evident.
	;   . s'il y a redondance il faut decider d'incrementer le compteur de
	;	redondance dans HARM_N_VIS (appele par RH_MALC_IM_2D, lui meme
	;	appele par CALC_IMAGE_HELIO) dans le cas de la nullite stricte)
	;	de la valeur des harmoniques.
	;   Proceder ainsi est plus facile que de definir des tableaux de per-
	;     mission d'utilisation d'antennes.
;  Tests de la gestion des harmoniques.
    toto = moment(indgen(3))	; pour compiler moment sans couper la liste.
    i_gestion = 0
    if ((n_image eq 1) and (i_gestion eq 1))then begin
	print, 'RH_CALC_IMAGE_HELIO , gestion des harmoniques :'
	if (correl(0, 18*25) eq  2)  then  begin
	    print, '    Donnees   : harm E0   * res NS,    ' + $
						'pas d harm NS45 * res NS'
	    i_e0_ns45 = 0		; E0 correle avec les NS
	endif
	if (correl(0, 18*25) eq 43)  then  begin
	    print, '    Donnees   : harm NS45 * res NS,    ' + $
						'pas d harm  E0  * res NS'
	    i_e0_ns45 = 45		; E0 correle avec les NS
	endif
	if (ie0_ew   eq 0) then  print, '    Suppression harm E0   * res EW'
	if (ie1_ew   eq 0) then  print, '    Suppression harm E1   * res EW'
	if (ie0_ns   eq 0) then begin	; E0 * res NS seulement avant jul_dat
	    if (correl(0, 18*25) eq 2)  then  $
			       print, '    Suppression harm E0   * res NS'
	endif
	if (ie2_ew   eq 0) then  print, '    Suppression harm E2   * res EW'
	if (ie2_ns   eq 0) then  print, '    Suppression harm E2   * res NS'
;     Tester si les harm de NS45 avec le reseau EW sont nuls ou non
	h45_ew = harm_N(432:449)
	mom = moment(h45_ew)
	c0 = complex(0, 0)
	if ( (mom(0) ne c0) and (mom(1) ne c0) ) then begin
	    ch_45_1 = '    harm NS45 * res EW non nuls dans les donnees'
	    ch_45_2 = ' '
	endif
	if ( (mom(0) eq c0) and (mom(1) eq c0) ) then begin
		; on admets que h45_ew est identiquement nul. Le if ne marche 
		;   pas sur des vecteurs.
		ch_45_1 = '    harm NS45 * res EW nuls dans les donnees'
		ch_45_2 = '    (deja nuls dans les donnees)'
	endif
	if (ins45_ew eq 1) then  print, ch_45_1
	if (ins45_ew eq 0) then  print, $
		   '    Suppression harm NS45 * res EW' + ch_45_2
;     Tester si les harm de NS45 avec le reseau NS sont nuls ou non
	h45_ns = harm_N(450:485)
	mom = moment(h45_ns)
	if ( (mom(0) ne c0) and (mom(1) ne c0) ) then begin
	    if (i_e0_ns45 eq  0) then $
		ch_45_1 = '    harm E0   * res NS non nuls dans les donnees'
	    if (i_e0_ns45 eq 45) then $
		ch_45_1 = '    harm NS45 * res NS non nuls dans les donnees'
	    ch_45_2 = ' '
	endif
	if ( (mom(0) eq c0) and (mom(1) eq c0) ) then begin
	    if (i_e0_ns45 eq  0) then $
		ch_45_1 = '    harm E0   * res NS '
	    if (i_e0_ns45 eq 45) then $
		ch_45_1 = '    harm NS45 * res NS '
	    ch_45_2 = '    deja nuls dans les donnees'
	endif
	if (ins45_ns eq 0) then  if (correl(0, 18*25) eq 43)  then  print, $
		   '    Suppression harm NS45 * res NS'
	if (ins45_ns eq 0) then  print, $
		   '    Suppression harm NS45 * res NS' + ch_45_2
    endif	
;  Fin des tests de gestion des harmoniques.

;   jul_ext = julday (10, 19, 1998)	; 19 oct 98.   ext comme "extension".
    jul_panne = julday (11, 27, 2009)	; 27 nov 09.   aa panne des antennes AA 
;    if n_elements(iaa_panne) eq 0 then iaa_panne = 0
    ipanne_correlateur = 0
    if jul_dat ge jul_panne then ipanne_correlateur = 1
    if ipanne_correlateur eq 1 then begin
        harm_N (624:628) = 0   ;  AA3 avec NS12  NS16
        harm_N (642:646) = 0   ;  AA4 avec NS12  NS16
;        print,' PANNE AA3 AA4: harm mis a 0'
     endif
    jul_panne2 = julday (06, 17, 2013)   ; 17 jun 13.   panne ns08 x ew01 
    if jul_dat ge jul_panne2 then begin
       num0 = where(rh_antenne(48) eq 'ns08')
       num1 = where(rh_antenne(48) eq 'ew01')
       ind_panne2 = where ( correl(0,*) eq num0(0) and correl(1,*) eq num1(0))
       harm_N(ind_panne2) = 0   ; ind_panne2 = 146
    endif
    if (ie0_ew eq 0) then   harm_N (522:539) = 0
    if (ie1_ew eq 0) then   harm_N (504:521) = 0
    if (ie0_ns eq 0) then begin		; E0 * res NS seulement avant jul_dat
	if (correl(0, 18*25) eq 2)  then  harm_N(450:485) = 0
    endif
    if (ie2_ew eq 0) then begin
	harm_N (486:503) = 0		; E2 avec H1 a H16.
	harm_N (504)     = 0		; E1 avec E2.
	harm_N (522)     = 0		; E0 avec E2.
    endif
    if (ie2_ns eq 0)   then   for i=0, 24   do  harm_N(18*i) = 0
    if (ins45_ew eq 0) then   harm_N(432:449) = 0
    if (ins45_ns eq 0) then  begin
	if (correl(0, 18*25) eq 43)  then  begin
	    harm_N(450:485) = 0
		    ; l'ant 43 est NS45. Quand la NS45 est utilisee (a partir 
		    ; du 19 oct 98 environ), les harmoniques NS45*resNS  sont 
		    ; a ces adresses, qui auparavant correspondaient a des cor-
		    ; relations E0 * res NS.
	    harm_N(572) = 0
	endif
    endif

;  Suppression des petites bases a partir de NS0 (le "gap")
    if (gap_ns0(0) ne 0) then begin
	gew = gap_ns0(0)
	harm_N(9-gew:9+gew) = 0.
    endif
    if (gap_ns0(1) ne 0) then begin
	gns = gap_ns0(1)
	for j = 0, gns  do begin
	    harm_N(9 + 18*j) = 0.
	endfor
    endif
;  Controle des harmoniques NS avec NSO_ew et NS0_ns (22 fev 2006)

;  Suppression des redondances 
    if (i_redond  eq  0) then begin
	harm_N(6:14) = 0    		; bases EW a partir de NS0 de -400m a 
					;    + 400m incluses.
	for j = 1, 6 do begin
	    harm_N(9 + 18*j) = 0	; NS0_ew avec NS1 a NS6
	endfor
	harm_N(540:544) = 0		; bases vers nord de NS8 avec NS0 a NS4
    endif

;  Suppression des harmoniques impliquant des antennes supposees a problemes
    d_0 = where(sel_ant eq 0, ic)		; numeros des ant en panne.
    if (ic ge 1) then begin
	if (n_image eq 1) then  $
	    print, 'RH_CALC_IMAGE_HELIO : mise a zero de tous les harm de ' + $
		'certaines antennes'
	for i=0, ic-1  do begin
	    for j=0, nbre_harm - 1   do begin
		if ((correl(0, j) eq d_0(i)) or (correl(1, j) eq d_0(i))) $
								then begin
		    harm_N(j) = 0
		    ch_format = "('    harm_N(', i3, ') = 0')"
;		    print, format=ch_format, j
		endif
	    endfor
	endfor
    endif

    ch_siz = 1.4		; taille des caracteres pour xyouts.

; Controle du niveau d'harmoniques polar : A VIRER
	; NS0  avec E2  a H16	  0   17
	; NS45 ---- ---------	432  449
	; ---- ---- les NS	450  485
	; NS8  ---- ------	540  557
	; NS0  ---- ------	558  575
    icon = 0
    if (icon eq 1) then begin
	toto = [harm_N(  0 :  17),  harm_N(432 : 449),  harm_N(450 : 485), $
		harm_N(540 : 557),  harm_N(558 : 575)]
	window, 0
	plot, abs(toto), xstyle=1
	xyouts, 0.5, 0.97, /normal, alignment=0.5, charsize=ch_siz, 'abs(toto)'
	print, 'abs(toto) ='
	print, abs(toto)
	stop
    endif

; Controle de la distribution des harmoniques dans "harm_N". 
    icon = 0			; 1 pour mise au point.
    if (icon eq 1) then begin
	bmp = reform(harm_N, nbre_harm)
	help, harm_N, bmp
	mod_amp = abs (bmp)
	help, mod_amp
	plot, mod_amp(0:35)
	        ; L'ordre de rangement des correlations est:	correlations
	        ;  NS0  avec E2, E1, H1 a H16		  	  0 a  17
	        ;  NS1  ---- ----------------------------      	 18 a  35
	stop
    endif
; Fin de controle.

; FIN DE SUPPRESSION EVENTUELLE DE CERTAINS HARMONIQUES.


; CALCUL DE L'IMAGE (NON POLAR OU POLAR) SUR LA GRILLE DES COORD INTERFEROME-
	; TRIQUES ENTIERES.

    if (i_simulation  eq  0) then begin		; donnees reelles.
	i_stop = 0				; 1 pour arrets controles
    endif
    if (i_simulation  gt  0) then begin		; donnees simulees.
	i_stop = 1				; 1 pour arrets controles
	    ; Rem du 25 sept 01 : RH_SIMUL_HARM_N a deja ete appel'e dans 
	    ;        RH_DPATCHFITS_NRH.
    endif

    i_stop = 0	    ; 1 pour pouvoir s'arreter aux controles de RH_MALC_IM_2D.

    RH_MALC_IM_2D, $
;     entree:
	i_stop,	i_sys_lin, $
	ch_polar,	   $
	i_pave_plein,  i_recentrage,  x_centrage,  y_centrage,	$
	freq,  ie0_ew,  harm_N,  correl,  npi,  n_bord,		$
;     sortie: image et lobe theorique, centre Soleil en (npi/2, npi/2).
	x_centrage_np,  y_centrage_np,				$
	tf_l_th,  l_th,  tf_im_int_s,  im_2d,  			$
	flux_int_s,  flux_compact,  uu,  vv
		;  uu, vv	coord de freq spatiales dans le plan (u, v),
		;		  ne servent qu'a des delouiseries.
		;  tf_l_th, tf_im_int_s  sont centrees en milieu de tableau.
		;		  ajoutes pour compatibilite le 13 mars 03.
;print,'max(im_2d)',max(im_2d)
    if max(im_2d) gt 1.e20 then begin
        print,' '
        formatbad = "(' Bad image: ',i10,' Freq:',f6.1,'  Max > 1e20:',g10.4)"
        print,n_image,freq,max(im_2d), format=formatbad
;        im_2d(*) = 0.
;        im_2d(npi/2-5:npi/2+5,npi/2-5:npi/2+5) = 1.
;        im_2d(npi/2,npi/2) = 1.5
;        flux_int_s = total(im_2d)
;stop
    endif


; Controle de l'image et de sa TF recalculee
    icon = 0
    if (icon eq 1) then begin
	taille = size(im_2d)
	np2 = taille(2) / 2
	tf_im_2d = shift(fft(shift(im_2d, -np2, -np2), 1), np2, np2)
	n_z = 32
	n1z = np2 - n_z      & n2z = np2 + n_z - 1
	toto = tf_im_2d(n1z : n2z,  n1z : n2z)
	toto_512 = congrid(abs(toto), 512, 512)
	window, 1
	loadct, 13
	tvscl, toto_512
	xyouts, 0.5, 0.95, /normal, alignment=0.5, charsize=1.3, $
	    'abs(visibilite recalculee d''apres image) apres RH_MALC_IM_2D'
	stop
	    ; Le 21 fev 2006, on constate que la redondance est bien corrigee
	    ;   et qu'il n'y a pas ce creux selon l'axe v.
	    ; On constate aussi que la composante continue a ete reconstituee.
	    ; Ca a l'air OK.
	    ; Le 23 frv 2006, on a aussi corrige une faute de copier-coller +
	    ;	faute de frappe dans la correction des harm de E0 dans 
	    ;   CORR_GAIN_ANT, qui entre autres, affectait les 8 premiers 
	    ;	harmoniques NS de NS8.
    endif

; Controle du niveau des harmoniques
    icon = 0
    if (icon eq 1)  then begin
	window, 0
	loadct, 0
;     Zooming
	nz = 15			; zooming de -nz a + nz
	n1 = npi/2 - nz	
	n2 = npi/2 + nz
	toto = abs(tf_im_int_s(n1:n2, n1:n2))
	shade_surf, toto
	xyouts, 0.5, 0.95, /normal, alignment=0.5, $
	    'abs(TF image, sortie de RH_MALC_IM_2D)
	stop
	    ; On trouve bien qque chose voisin de 1 pour le Cygne.
    endif	; Fin controle du niveau des harmoniques.

; Controle du 17 oct 2005 (mise au point de la synthese RH + GMRT)
	; a permis de trouver qu'il fallait que FILTRE_PARASITES conserve la
	; dimension de harm_N.
    icon = 0
    if (icon eq 1)  then begin
	i_sh_tv = 2		; 1 shade_surf,  2 tvscl
	if (i_sh_tv  eq  1) then begin
	    window, 1
	    shade_surf, congrid (im_2d, 512, 512, cubic=-0.5)
	endif
	if (i_sh_tv  eq  2) then begin
	    window, 1, xsize=512, ysize=512
	    tvscl, congrid (im_2d, 512, 512, cubic=-0.5)
	endif
	xyouts, 0.5, 0.95, /normal, alignment=0.5, charsize=1.3, $
	    'Image apres sortie de RH_MALC_IM_2D dans RH_CALC_IMAGE_HELIO'
	help, tf_l_th,  l_th,  tf_im_int_s,  im_2d,  			$
		flux_int_s,  flux_compact,  uu,  vv
	stop
    endif	; Fin controle image finale.

; Verification des flux totaux et compacts
    icon = 0
    if (icon eq 1) then begin
	print, 'RH_CALC_IMAGE_HELIO : sortie de RH_MALC_IM_2D'
	ch_format ="(4x, 'flux total   =', f7.2, ' sfu (par somme des " + $
					"bordures nulle + amelioration)')"
	print, format=ch_format, flux_int_s
	ch_format ="(4x, 'flux compact =', f7.2, ' sfu (extrapolation " + $
					"quadratique de la visibilite)')"
	print, format=ch_format, flux_compact
	stop
    endif

; Verification de im_2d et l_th (controle servant a voir l'effet du remplissa-
	; ge du pave. A permis de trouver qu'il fallait que FILTRE_PARASITES 
	; conserve la dimension de harm_N)
    icon = 0
    if (icon eq 1) then begin
	print, 'RH_CALC_IMAGE_HELIO : sortie de RH_MALC_IM_2D, avant clean'
	im_2d_512 = congrid(im_2d, 512, 512, cubic=-0.5)
	l_th_512  = congrid(l_th , 512, 512, cubic=-0.5)
	i_sh_tv = 2		; 1 shade_surf, 2 tvscl.
;     Trace du lobe
	if (i_sh_tv  eq  1) then begin
	    window, 0
	    shade_surf, l_th_512
	endif
	if (i_sh_tv  eq  2) then begin
	    window, 0, xsize=512, ysize=512
	    tvscl, l_th_512
	endif
	if (i_pave_plein eq 0) then ch_2 = ', pave lacunaire'
	if (i_pave_plein gt 0) then ch_2 = ', pave plein'
	xyouts, 0.5, 0.976, /normal, alignment=0.5, $
				'l_th (sortie de malc_im_2d)' + ch_2
;     Trace de l'image
	if (i_sh_tv  eq  1) then begin
	    window, 1
	    shade_surf, im_2d_512
	endif
	if (i_sh_tv  eq  2) then begin
	    window, 1, xsizeize=1512, ysize=512
	    tvscl, im_2d_512
	endif
	xyouts, 0.5, 0.976, /normal, alignment=0.5, charsize=ch_siz, $
				'im_2d (sortie de malc_im_2d)' + ch_2
	stop
    endif

; Verification l_th (voir si on utilise NS45)
    icon = 0
    if (icon eq 1) then begin
	window, 0
	shade_surf, shift(abs(fft(l_th, 1)), npi/2, npi/2)
	stop
    endif

	; . correl est le tableau des antennes correlees pour les nbre_harm 
	;     harm. Cest un intarr(2, nbre_harm).
	; . ie0_ew ne sert qu'a choisir les bas harmoniques utilises dans
	;	  le calcul de la composante continue.
	; . l_th est le lobe theorique centr'e en (npi/2, npi/2), obtenu
	;     en mettant a 1 l'amplitude des harmoniques (c'est a dire, 
	;     etant donnee la procedure d'etalonnage, comme s'ils etaient 
	;     obtenus avec une source ponctuelle de 1 sfu). Il est normalise
	;     dans RH_MALC_IM_2D pour que son max soit egal 1. Cette normali-
	;     sation est pratique pour l'usage qui en est fait dans P_CLEAN_2D
	;     (dans WCLEAN) : retrait de composantes clean de l'image residu-
	;     elle. La normalisation est donc differente de celle de im_2d 
	;     depuis le 11 dec 00.
	; . im_2d est l'image interferometrique, non polar ou polar, (centree 
	;     en npi/2, npi/2) calculee sur la grille des coord interferometri-
	;     ques r et s entieres, et sur un champ interferometrique entier. 
	;     L'image pour r=rg et s=sg (non entiers) sera interpolee plus bas
	;     a partir de l'image sur la grille des coord interfero entieres.
	;     A la difference de l_th, im_2d est normalisee de sorte que la 
	;     somme de ses points soit le flux en sfu. 
	; . pour des sources non solaires, les axes sont orientes
	;	   . vers W (ascensions droites < celle du centre) et 
	;	   . vers N (declinaisons < celle du centre), 
	;	   car l'angle P a ete pris egal a 0 par INIT_MALAX (de meme 
	;	   que les derives en angle horaire et declinaison).
	;      Les echelles angulaires sont les memes sur les 2 axes.
	; . uu, vv   coord de frequences spatiales dans le plan (u, v).

; Verif de dom_c
    icon = 0
    if (icon  eq  1) then begin
	im_2d_512 = congrid(im_2d, 512, 512, cubic=-0.5)
	toto = im_2d
	toto(dom_c) = 0
	toto_512 = congrid(toto, 512, 512, cubic=-0.5)
	window, 0
	tvscl, im_2d_512
	window, 1
	tvscl, toto_512
	stop
    endif

; Sauvegarde de l'image initiale (avec ou sans clean)
    im_int_s = im_2d	; image interfero sale.

; FIN DE CALCUL D'IMAGE (NON POLAR OU POLAR) SUR LA GRILLE DES COORD INTERFE-
	; ROMETRIQUES ENTIERES.

; Controle ajoute le 17 oct 2005 (a permis de trouver que FILTRE_PARASITES
	; devait conserver la dimension de harm_N pour que RH_HARM_N_VIS
	; puisse marcher lors de la recherche des sursauts pour la synthese 
	; RH + GMRT)
    icon = 0
    if (icon eq 1) then begin
	print, 'RH_CALC_IMAGE_HELIO : sortie de RH_MALC_IM_2D, avant clean'
	im_int_s_512 = congrid(im_int_s, 512, 512, cubic=-0.5)
	l_th_512     = congrid(l_th    , 512, 512, cubic=-0.5)
	i_sh_tv = 2		; 1 shade_surf, 2 tvscl.
;     Trace du lobe
	if (i_sh_tv  eq  1) then begin
	    window, 0
	    shade_surf, l_th_512
	endif
	if (i_sh_tv  eq  2) then begin
	    window, 0, xsize=512, ysize=512
	    tvscl, l_th_512
	endif
	if (i_pave_plein eq 0) then ch_2 = ', pave lacunaire'
	if (i_pave_plein gt 0) then ch_2 = ', pave plein'
	xyouts, 0.5, 0.976, /normal, alignment=0.5, $
				'l_th (sortie de malc_im_2d)' + ch_2
;     Trace de l'image
	if (i_sh_tv  eq  1) then begin
	    window, 1
	    shade_surf, im_int_s_512
	endif
	if (i_sh_tv  eq  2) then begin
	    window, 1
	    tvscl, im_int_s_512
	endif
	xyouts, 0.5, 0.976, /normal, alignment=0.5, charsize=ch_siz, $
				'im_int_s  (sortie de malc_im_2d)' + ch_2
	stop
    endif	; Fin de controle ajoute le 17 oct 2005.

; Controle rustine pour voir l'image et abs de sa TF
    icon = 0
    if (icon eq 1) then begin
	print, 'RH_CALC_IMAGE_HELIO : sortie de RH_MALC_IM_2D, avant clean'
	im_int_s_512 = congrid(im_int_s, 512, 512, cubic=-0.5)
;     Trace de l'image
	window, 0
	loadct, 0
	shade_surf, im_int_s_512
;	tvscl, im_int_s_512
	xyouts, 0.5, 0.976, /normal, alignment=0.5, charsize=ch_siz, $
				'im_int_s  (sortie de malc_im_2d)'
;     Trace de abs(TF image), avec zoom
	dnz = 32
        n1z = 64 - dnz    & n2z = 64 + dnz - 1
	toto_z = tf_im_int_s (n1z : n2z, n1z : n2z) 
	toto_512 = congrid(abs(toto_z), 512, 512)
	loadct, 13
	window, 1
	tvscl, toto_512
	loadct, 0
	xyouts, 0.5, 0.976, /normal, alignment=0.5, charsize=ch_siz, $
		'abs(im_int_s)  en sortie de malc_im_2d'
	stop
    endif	; Fin de controle ajoute le 17 oct 2005.


; CLEAN
    t0 = systime(1)			; temps en sec depuis le 1er jan 70.
    if (mode_clean  eq  0)  then begin
	im_int_p = im_int_s
	tf_lobe_synt = tf_l_th
	lobe_synt_int = l_th
    endif
    if (mode_clean  gt  0)  then begin
;     Controle
	icon = 0
	if (icon eq 1) then begin
	    print, 'avant appel RH_CLEAN'
	    shade_surf, congrid(image, 512, 512, cubic=-0.5)
	    stop
	endif

	crit_arret_2 = crit_arret
	if (ch_polar  eq  'polar') then  crit_arret_2 (1) = 0
	    ; on n'utilise pas le citere d'arret sur le flus en polar.

	RH_CLEAN, $
	 ; entree
	    ; donnees image :
		freq, ch_polar,						$
		i_simulation,     i_pave_plein, 			$
		tf_objet_simule,  objet_simule,  			$
		tf_im_int_s,  im_int_s,  tf_l_th,  l_th, 		$
	    ; clean :
		mode_clean,   dom_c,   dom_a_i,				$
		i_ech_min,  i_ech_max ,  a_in,  a_ex,			$
		a_red_fil,  itermax,  gain_iter,  jmax,  crit_arret_2,	$
		i_seuil, coeff_seuil, frac_flux, fac_crois_res, 	$
		i_test_arret,  i_lissage,   poids_c,  larg_2,		$
	 ; sortie :
	   tf_lobe_synt_int,  lobe_synt_int,  im_int_p,			$
	   objet_simule_lisse

	    ; im_int_p	     image interferometrique propre
	    ; lobe_synt_int  lobe propre interferometrique (TF (sigma_fliltre *
	    ;			filtre final) ).
	    ;
	    ; Remarques :
	    ; . tf_im_int_s  et  tf_l_th  sont centrees en milieu de tableau.
	    ; . suppression du mot-cle "self" dans la liste d'entree.  "self" 
	    ;     qui n'est pas defini en amont et ne sert qu'a des delouise-
	    ;	  ries en aval.
	    ; . correl, champ et harm_N ne servent qu'a des delouiseries obscu-
	    ;	  res dans RH_CLEAN (mises en commentaires), et n'apparait pas
	    ;	  dans WCLEAN.

	im_2d = im_int_p	; image interfero propre.

;     Verification du flux apres CLEAN
	icon = 0
	if (icon eq 1) then begin
	    print, 'RH_CALC_IMAGE_HELIO : flux des images interferometriques'
	    print, '  avant CLEAN'
	    ch_format = "(4x, 'flux total   =', f7.2, ' sfu (somme " + $
				"des des point image interfero sale)')"
	    print, format=ch_format, flux_int_s
	    print, '                      (base d''image par somme des ' + $
			'bordures nulle + amelioration)'
	    ch_format ="(4x, 'flux compact =', f7.2, ' sfu (par extrapola" + $
				"tion quadratique de visibilite)')"
	    print, format=ch_format, flux_compact
	    print, '  apres CLEAN'
	    ch_format = "(4x, 'flux total   =', f7.2, ' sfu (somme des " + $
				"points image interfero propre)')"
	    print, format=ch_format, total(im_int_p)
		; Rappel :  flux_int_s  et  flux_compact  (en sfu) sont four-
		;		nis en sortie de RH_MALC_IM_2D. 
	    stop
	endif
   endif		; fin de clean.

; Mesure du temps de calcul pour le clean
    t1 = systime(1)
    ch_1 = "('Temps de clean', "
    ch_2 = "' sec')"
    ch_format = ch_1 + " f10.3," + ch_2
    temps_clean = t1 - t0
;   print,  format=ch_format,  t1 - t0	; impression genante.

; Fin du clean

; Ecriture image interferometrique propre  
	; L'image helio est ecrite en fin de CALC_IMAGE_HELIO, eventuellement 
	; ecretee a des niveaux pour voir des details faibles, en meme temps
	; que sont affiches les images interfero sales et propres, helio sale 
	; et propre.
    i_ecrit = 0
    if(i_ecrit eq 1) then begin
	im_int_p_512 = congrid(im_int_p, 512, 512, cubic=-0.5)
	ch_code = string(format='(i2)', fix(100*a_red_fil))
	    ; a_red_fil est le facteur d'homothetie pour reduire l'extension 
	    ;   des zones de filtrage d'echelle, defini dans dpnew.
;	n_fich = 'im_' + ch_code + '.gif'
	n_fich = 'im_cub_plein_propre_1.gif'
	    ; lacun_sale    lacun_propre    plein_sale    plein_propre
	    ; im_68.gif   pour a_red_fil=0.68.
	write_gif, n_fich, bytscl(im_int_p_512)	; sur /home/mercier/cleanself/.
	print, 'RH_CALC_IMAGE HELIO : ecriture gif image interfero propre'
	window, 2
	shade_surf, im_int_p_512
	xyouts, 0.5, 0.97, /normal, alignment=0.5, charsize=ch_siz, $
		'Image interfero finale, ecrite en fichier gif'
	stop
    endif

; DEBUT DE CONTROLES LIES AU CLEAN OU NON

; Controle: visualisation du lobe theorique et de l'image interfero propre
    icon = 0
    if (icon eq 1) then begin
	l_th_512     = congrid(l_th    , 512, 512, cubic=-0.5)
	im_int_p_512 = congrid(im_int_p, 512, 512, cubic=-0.5)
	tvscl, l_th_512
	xyouts, 0.4, 0.96, 'lobe theorique', alignment=0.5, charsize=ch_siz
	window, 1
	tvscl, im_int_p_512
	xyouts, 0.4, 0.96, 'image sortie RH_MALC_IM_2D', alignment=0.5, $
			charsize=ch_siz
	stop
	window, 0
	plot, l_th_512(*, 256)
	window, 1    
	plot, l_th_512(256, *)
	stop
    endif

; Controle: localisation du flux du lobe theorique
    icon = 0
    if (icon eq 1) then begin
	Flux_c = fltarr(npi/2)			; _c comme coup'e.
	for j = 1, npi/2-1  do begin
	    l_th_c = l_th(npi/2 - j : npi/2 + j,    npi/2 - j : npi/2 + j)
	    Flux_c(j) = total(l_th_c)
	endfor
	l_th_512 = congrid(l_th, 512, 512, cubic=-0.5)
;     Traces de coupes EW et NS du lobe
	plot, l_th_512(*, 256)		; coupe EW
	window, 1
	plot, l_th_512(256, *)		; coupe NS
	stop
;     Visualisation du lobe et trace du flux vs largeur d'integration.
	window, 0
	tvscl, l_th_512
	window, 1
	plot, Flux_c, xrange=[1, npi/2], xstyle=1
	stop
    endif
; 	=>  il manque 20% du flux si on integre l_th sur 1/4 central du champ.
;	     C'est du a la presence des harmoniques purement EW impairs dus a
;	     E0. L'effet disparait et le flux du lobe est bien localise si on
;	     les supprime. Conserver les harm de E2 avec l'EW et les NS ne 
;	     detruit pas la localisation du flux.
; Fin de controle.

; Controle: localisation du flux d'un lobe theorique elargi par gaussienne.
    icon = 0
    if (icon eq 1) then begin
	pond = fltarr(npi, npi)
	nh = 10	; numero de l'harm attenue par un facteur 2.
	ak = alog(2)/nh^2	; coeff de i^2 et j^2 dans la gaussienne. 
	for i = -npi/2, npi/2-1  do begin		; boucle sur colonnes
	    for j = -npi/2, npi/2-1  do begin		; boucle sur lignes
		pond(npi/2+i, npi/2+j) = exp(-ak * (i^2 + j^2))
	    endfor
	endfor
;     Visualisation ponderation visibilite, des lobes theo et elargi. 
	pond = shift(pond, -npi/2, -npi/2) ; ponderateur centre au coin inf g.
;	shade_surf, congrid(pond, 512, 512, cubic=-0.5)
;	stop
	im_gauss = fft( (pond * fft(l_th, 1)),   -1)
;	tvscl, congrid(l_th, 512, 512, cubic=-0.5)
	window, 1
;	tvscl, congrid(im_gauss, 512, 512, cubic=-0.5)
;	stop
;     Calcul et visualisation du flux partiel.
	Flux_c = fltarr(npi/2)
	for j = 1, npi/2-1  do begin
	    im_gauss_c = $	; reduction au centre
		im_gauss(npi/2 - j : npi/2 + j,    npi/2 - j : npi/2 + j)
	    Flux_c(j) = total(im_gauss_c)
	endfor
	window, 0
	tvscl, congrid(im_gauss, 512, 512, cubic=-0.5)
	window, 1
	plot, Flux_c, xrange=[1, npi/2], xstyle=1
	print, 'Numero de l harm attenue par un facteur 2:', nh
	stop
    endif
; 	=>  resultat identique a celui de la localisation du flux du lobe theo-
;	    rique: ce sont les harmoniques de E0 qui detruisent la localisation
;	    du flux.
; Fin de controle.

; FIN DE DIVERS CONTROLES LIES AU CLEAN OU NON


; DEBUT DELOUISERIE
; Normalisation de l'image interferometrique non polar facon J-M (obsolete 
	; depuis que c'est fait dans RH_MALC_IM_2D.
;   norm_np2 = total (CALC_CBEAM(l_th))	; version J-M. 
;   ; CALC_CBEAM retourne (en fltarr(npi, npi)) une gaussienne a sup-
;	    ;   port elliptique qui represente au mieux le lobe theorique l_th.
;	    ;   Le modele est centre au centre du champ.
;	    ; Remarque: J-M totalise CALC_CBEAM(l_th) qui a necessairement une
;	    ;		  composante continue (approximative...), alors que 
;	    ;		  l_th n'en a pas dans la version originale de J-M, 
;	    ;		  sans restauration de composante de Fourier zero.
;   im_2d = im_2d / norm_np2		; version J-M.

;   if (t_b eq 1) then  sfu_tb = 1.			; image en unites sfu.
;   if (t_b eq 2) then  sfu_tb = NORM_TBMA (ma, freq)	; image en Tb.
; FIN DELOUISERIE


; PERIODISATION DE L'IMAGE INTERFEROMETRIQUE
    if (n_period eq 1) then begin
        im_int_s_etendue = im_int_s		; non cleanee
        im_int_p_etendue = im_int_p		; cleanee
	l_th_etendu      = l_th
	l_sy_etendu	 = lobe_synt_int
    endif
    if (n_period gt 1) then begin
	RH_PERIODISE, n_period, npi, im_int_p,	      $	; entree
		      im_int_p_etendue			; sortie
	RH_PERIODISE, n_period, npi, im_int_s,	      $	; entree
		      im_int_s_etendue			; sortie
	RH_PERIODISE, n_period, npi, l_th,            $	; entree
		      l_th_etendu			; sortie
	RH_PERIODISE, n_period, npi, lobe_synt_int,   $	; entree
		      l_sy_etendu			; sortie
		; im_int_p_etendue,  l_th_etendu,  l_sy_etendu  sont definis 
		;    sur npi*n_period points.
    endif

; Controle de l'image interferometrique etendue
    icon = 0
    if (icon eq 1) then begin
	window, 0
	tvscl, congrid(im_int_p, 512, 512, cubic=-0.5)
	xyouts, 0.5, 0.97, /normal, alignment=0.5, charsize=ch_siz, $
		'image interfero non periodisee apres clean'
	window, 1
	tvscl, congrid(im_int_p_etendue, 512, 512, cubic=-0.5)
	xyouts, 0.5, 0.97, /normal, alignment=0.5, charsize=ch_siz, $
		'image interferometrique etendue'
	stop
    endif

; FIN DE PERIODISATION

; CALCUL DE L'IMAGE HELIOGRAPHIQUE FINALE (soleil rond) par:
	; - interpolation de l'image interferometrique,
	; - normalisation:
	;   . si l'image est en brillance, on la normalise de sorte que son 
	;	flux soit simplement la somme de ses np*np points.
	;   . si l'image est en temp. de brillance, elle est en Kelvins.
    t0 = systime(1)			; temps en sec depuis le 1er jan 70
    if (mode_interpol  eq  1) then begin
;	lobe_sale_hel = interpolate (l_th            ,  rg,  sg,  miss=0)
	    ; faute trouvee le 2 mars 06 : rg et sg s'appliquant a un champ
	    ;   etendu, il faut que le lobe soit etendu aussi
	lobe_sale_hel = interpolate (l_th_etendu     ,  rg,  sg,  miss=0)
	im_hel_s      = interpolate (im_int_s_etendue,  rg,  sg,  miss=0)
	if (mode_clean  eq  0) then begin
	    lobe_synt_hel = lobe_sale_hel
	    im_hel_p = im_hel_s
	endif
	if (mode_clean  gt  0) then begin
	    lobe_synt_hel = interpolate (l_sy_etendu     ,  rg,  sg,  miss=0)
	    im_hel_p      = interpolate (im_int_p_etendue,  rg,  sg,  miss=0)
		; rappel : l_sy_etendu  est le lobe synthetique interferometri-
		;	    que propre etendu. soit l'equivalent "propre" de
		;	    l_th_etendu.
	endif
	    ; miss=val fait que les positions interpolees au dela des limites
	    ;   de im_int_p_etendue (ici rg<0 ou rg>n_period*npi-1) sont mises 
	    ;   a "val" (sinon elles reproduiraient les valeurs extremes d'en-
	    ;   tree).
	    ; Remarques :
	    ; . "miss=0" fait le meme effet que "champ".
	    ; . la rotation d'angle p a ete inclus en amont dans le calcul de
	    ;	  rg  et  sg.

    endif   ; fin de if (mode_interpol  eq  1)
    if (mode_interpol  gt  1) then begin
	    ; On commence par interpoler l'image d'un facteur 2 ou 4 avec une 
	    ;   bonne interpolation, puis on utilise une interpolation lineai-
	    ;   re.
	np_etendu = npi * n_period	
	    ; nbre de pts existant deja sur l'image interferometrique etendue.
	fac = 2
	rg_fac = rg * fac
	sg_fac = sg * fac
	np_et_fac = np_etendu * fac
	if (mode_interpol  eq  2) then begin	; interpolation par facteur fac
;	    l_sa_int_fac = congrid(l_th            , npi * fac, npi * fac, $
		; faute trouvee le 22 mars 2006 : tout doit etre "etendu".
	    l_sa_int_fac = congrid(l_th_etendu     , np_et_fac, np_et_fac, $
								cubic=-0.5)
	    im_int_s_fac = congrid(im_int_s_etendue, np_et_fac, np_et_fac, $
								cubic=-0.5)
	    if (mode_clean  eq  0) then begin
		l_sy_int_fac = l_sa_int_fac
		im_int_p_fac = im_int_s_fac
	    endif
	    if (mode_clean  gt  0) then begin
		l_sy_int_fac = congrid(l_sy_etendu     , np_et_fac, np_et_fac,$
								cubic=-0.5)
		im_int_p_fac = congrid(im_int_p_etendue, np_et_fac, np_et_fac,$
								cubic=-0.5)
	    endif
		; Remarques :
		; . im_int_p_etendue est definie meme si mode_clean=0, auquel
		;     cas elle est identique a  im_int_s_etendue.
		; . rebin ne fait qu'une interpolation bilineaire.
		; . on interpole les lobes (sale et synthetique) comme l'image,
		;     mais sans les etendre a  n_period  periodes, ce qui est
		;     est inutile pour les incrustations.
	endif   ; fin de  if (mode_interpol  eq  2) 
	if (mode_interpol eq 3) then begin
	    l_sa_int_fac = RH_INTER_FFT (l_th_etendu     , np_et_fac)
	    im_int_s_fac = RH_INTER_FFT (im_int_s_etendue, np_et_fac)
	    if (mode_clean  eq  0) then begin
		l_sy_int_fac = l_sa_int_fac
		im_int_p_fac = im_int_s_fac
	    endif					; icici
	    if (mode_clean  gt  0) then begin
		l_sy_int_fac = RH_INTER_FFT (l_sy_etendu     , np_et_fac)
		im_int_p_fac = RH_INTER_FFT (im_int_p_etendue, np_et_fac)
	    endif
	endif
;     Interpolation lineaire finale sur l'image deja interpolee du facteur 
	    ; d'interpolation entier  "fac".  Rappel rg_fac = rg * fac.
	im_hel_s      = interpolate (im_int_s_fac,  rg_fac,  sg_fac,  miss=0)
	lobe_sale_hel = interpolate (l_sa_int_fac,  rg_fac,  sg_fac,  miss=0)
	im_hel_p      = interpolate (im_int_p_fac,  rg_fac,  sg_fac,  miss=0)
	if (mode_clean  eq  0) then begin
	    lobe_synt_hel = lobe_sale_hel
	    im_hel_p      = im_hel_s
	endif
	if (mode_clean  gt  0) then begin
	    lobe_synt_hel = interpolate (l_sy_int_fac,  rg_fac, sg_fac, miss=0)
	    im_hel_p      = interpolate (im_int_p_fac,  rg_fac, sg_fac, miss=0)
	endif
	    ; les limites au dela desquelles im_int_p est mise a zero sont
	    ;   maintenant 0 et np_et_fac.
	    ; rappel :
	    ; . "miss=0" fait le meme effet que "champ".
	    ; . la rotation d'angle p a ete inclus en amont dans le calcul de
	    ;	  rg  et  sg.
    endif	; fin de  if (mode_interpol  gt  1).
    t1 = systime(1)
    ch_1 = "('Calcul image heliographique : temps interpolation "
    if (mode_interpol eq 1) then ch_2 = "bilineaire ='"
    if (mode_interpol eq 2) then ch_2 = "spline ='"
    if (mode_interpol eq 3) then ch_2 = "FFT =  '"
    ch_4 = "' sec')"
    ch_format = ch_1 + ch_2 + ", f6.3," + ch_4
    temps_interpol =  t1 - t0
;   print,  format=ch_format,  temps_interpol		; impression genante.
    temps_total = temps_clean + temps_interpol
    ch_format = '("RH_CALC_IMAGE_HELIO, temps total de calcul d''image =", ' +$
	'f5.2, " sec")'
;;   print, format=ch_format, temps_total

    ; Rem : im_hel_s et im_hel_p sont ici les images heliographiques sales et 
    ;		propres de np*np points. L'interpolation par le facteur fac
    ;		(usuel 2) a disparu du fait de l'usage de rg_fac et sg_fac.

; FIN DE CALCUL D'IMAGE HELIOGRAPHIQUE IM_2D PAR INTERPOLATION

; Controle des lobes heliographiques sale ou synthetique (propre) sur le champ
	; np*np
    icon = 0
    if (icon eq 1) then begin
	i_choix_lobe = 2		; 1 lobe sale, 2 lobe synthetique.
	j_sh_tv = 2
	if (i_choix_lobe  eq  1) then begin
	    lobe_int = l_th
	    lobe_hel = lobe_sale_hel
	    ch_int = 'lobe interferometrique sale'
	    ch_hel = 'lobe heliographique    sale'
	endif
	if (i_choix_lobe  eq  2) then begin
	    lobe_int = lobe_synt_int
	    lobe_hel = lobe_synt_hel
	    ch_int = 'lobe interferometrique propre'
	    ch_hel = 'lobe heliographique    propre'
	endif
;      Trace lobe intererometrique
	help, lobe_int, lobe_hel
	print, 'max(lobe_int) =', max(lobe_int)
	print, 'max(lobe_hel) =', max(lobe_hel)
	if (j_sh_tv  eq  1) then begin
	    window, 0
	    loadct, 0
	    shade_surf, lobe_int
	endif
	if (j_sh_tv  eq  2) then begin
	    window, 0, xsize=512, ysize=512
	    loadct, 13
	    tvscl, congrid(lobe_int, 512, 512, cubic=-0.5)
	endif
	loadct, 0
	xyouts, 0.5, 0.95, /normal, alignment=0.5, charsize=1.3, ch_int
;      Trace lobe heliographique
	if (j_sh_tv  eq  1) then begin
	    window, 1
	    loadct, 0
	    shade_surf, lobe_hel
	endif
	if (j_sh_tv  eq  2) then begin
	    window, 1, xsize=512, ysize=512
	    loadct, 13
	    tvscl, congrid(lobe_hel, 512, 512, cubic=-0.5)
	endif
	loadct, 0
	xyouts, 0.5, 0.95, /normal, alignment=0.5, charsize=1.3, ch_hel
	stop
	    ; tvscl, congrid(im_int_p, 512, 512, cubic=-0.5)
	    ; tvscl, congrid(im_hel_p, 512, 512, cubic=-0.5)
    endif
;----------------------------------------------------------
    if i_affich_lobe eq 0 then goto, m1
; Construction de vignettes contenant les sections a mi-puissance des lobes 
	;   heliographiques sales et propre (synthetique), pretes a etre 
	;   incrustees sur les images heliographiques sales et propre :
	;   . np *np   pour ecriture fits par RH_DPATCHFITS_NRH) et
	;   . 512*512  pour affichage de controle dans RH_CALC_IMAGE_HELIO
	; Il semble que le lobe synthetique helio (image definie sur np points)
	;   soit toujours inscrit dans un tableau carre de dimension 
	;	50 * (164 / freq) * (np / 128) * (4 / champ_helio)
	;   avec   champ_helio  en  Rs  et  freq  en MHz
	; Ce tableau est plus petit en ete et pres du meridien
    dim_164 = 60.0
	; On constate qu'il faut un cadre de dimension ~ 55 pour contenir le 
	;   lobe d'hiver a 164 MHz pour une image 128 * 128.
	; On reduira l'encombrement un peu plus bas.
    n_z_ls = nint(dim_164 * (164.0 / freq) * (float(np) / 128) * $
						(4.0 / champ_helio) / 2) * 2
	; dimension du tableau carre pouvant contenir le lobe. Doit etre paire
	; car on fabrique d'abord un tableau de dimemsion  (n_z_ls / 2) * 2,
	; qu'on considere ensuite etre de dimension n_z_ls, et cela doit evide-
	; mente tre coherent.
;  Lobe sale "cadr'e serr'e"
;   Recherche du max sur une zone centrale de l'image
;    la recherche sur l'image complete pouvant donner le max sur les
;    alias du pic du lobe (avant implantation des antennes anti-alias)
;    ce qui plantait le prog (Bug corrige le 23 aout 2007)
    lobe_temp = lobe_sale_hel
    lobe_temp(*) = 0.
    i0 = np / 2 - n_z_ls / 2
    i1 = np / 2 + n_z_ls / 2 -1
    lobe_temp(i0:i1,i0:i1)= lobe_sale_hel(i0:i1,i0:i1)
    lobe_sale_hel_max = max(lobe_temp,ipm)
    j_li_m = ipm / np
    i_co_m = ipm  mod  np
    n1x_ls = i_co_m  -  n_z_ls / 2      & n2x_ls = i_co_m  +  n_z_ls / 2 - 1
    n1y_ls = j_li_m  -  n_z_ls / 2      & n2y_ls = j_li_m  +  n_z_ls / 2 - 1
    lobe_sale_hel_red = lobe_sale_hel (n1x_ls : n2x_ls,  n1y_ls : n2y_ls)
;stop
;  Lobe synthetique "cadr'e serr'e"
    if (mode_clean  gt  0) then begin
	lobe_synt_hel_max = max(lobe_synt_hel,ipm)
	j_li_m = ipm / np
	i_co_m = ipm  mod  np
	n1x_ls = i_co_m  -  n_z_ls / 2      & n2x_ls = i_co_m  +  n_z_ls / 2 -1
	n1y_ls = j_li_m  -  n_z_ls / 2      & n2y_ls = j_li_m  +  n_z_ls / 2 -1
	lobe_synt_hel_red = lobe_synt_hel (n1x_ls : n2x_ls,  n1y_ls : n2y_ls)
    endif

; Section du lobe sale heliographique pour une image  np*np  points 
    s_lo_sa_h = fltarr (n_z_ls,  n_z_ls)
    dom_sec = where(lobe_sale_hel_red  ge  0.5 * max(lobe_sale_hel_red), ip)
	; on prend "ge" plutot que "gt" car il vaut mieux un lobe un peu trop
	; grand et aussi moins pixellis'e.
    s_lo_sa_h (dom_sec) = 1
;  Reduction a un champ ~ 2 fois plus etroit que celui du lobe sale 
    fac_red = 0.60			; 0.60 pour etre un peu large
    n_z_ls_2 = nint(fac_red * n_z_ls / 2)
    ns1 = n_z_ls / 2  -  n_z_ls_2
    ns2 = n_z_ls / 2  +  n_z_ls_2
    s_lo_sa_h = s_lo_sa_h (ns1 : ns2,  ns1 : ns2)
; Section du lobe synthetique pour une image  np*np  points
	; On ne fait rien pour la section du lobe synthetique si  mode_clean=0,
	;   on la prendra simplement egale a celle du lobe sale apres reduction
	;   de l'encombrement un peu plus bas.
    if (mode_clean  gt  0) then begin
	; avec la dimension convenant a des images 512*512 (pour eviter les 
	; artefacts <0 d'interpolation) lors de l'afichage de controle.
	s_lo_sy_h = fltarr (n_z_ls,  n_z_ls)
	dom_sec = where(lobe_synt_hel_red  ge  0.5 * max(lobe_synt_hel_red),ip)
	    ; on prend "ge" plutot que "gt" car il vaut mieux un lobe un peu 
	    ;   trop grand et aussi moins pixellis'e.
	s_lo_sy_h (dom_sec) = 1
;  Reduction a un champ ~ 2 fois plus etroit que celui du lobe synthetique
	n_z_ls_2 = nint(fac_red * n_z_ls / 2)
	ns1 = n_z_ls / 2  -  n_z_ls_2
	ns2 = n_z_ls / 2  +  n_z_ls_2
	s_lo_sy_h = s_lo_sy_h (ns1 : ns2,  ns1 : ns2)
    endif
; Reduction des sections des lobes heliographiques pour images  np*np  a des 
	; rectangles moins encombrants
;  Lobe sale
    som_sa_1 = reform(total(s_lo_sa_h, 2))	; somme sur 1er  indice
    som_sa_2 = reform(total(s_lo_sa_h, 1))	; somme sur 2eme indice
	; rem : les indices sont "croises" ci dessus !.
    dom_1 = where(som_sa_1  gt  0, ic1)
    dom_2 = where(som_sa_2  gt  0, ic2)
    ix1 = min(dom_1) - 3   & if (ix1  lt             0) then ix1 = 0
    ix2 = max(dom_1) + 3   & if (ix2  gt  2 * n_z_ls_2) then ix2 = 2 * n_z_ls_2
    iy1 = min(dom_2) - 3   & if (iy1  lt             0) then iy1 = 0
    iy2 = max(dom_2) + 3   & if (iy2  gt  2 * n_z_ls_2) then iy2 = 2 * n_z_ls_2
    s_lo_sa_h = s_lo_sa_h (ix1 : ix2,  iy1 : iy2)
;  Lobe synthetique
    if (mode_clean eq  0) then  s_lo_sy_h = s_lo_sa_h
    if (mode_clean gt  0) then begin
	som_sy_1 = reform(total(s_lo_sy_h, 2))	; somme sur 1er  indice
	som_sy_2 = reform(total(s_lo_sy_h, 1))	; somme sur 2eme indice
	dom_1 = where(som_sy_1  gt  0, ic1)
	dom_2 = where(som_sy_2  gt  0, ic2)
	ix1 = min(dom_1) - 5   & if (ix1  lt           0) then ix1 = 0
	ix2 = max(dom_1) + 4   & if (ix2  gt  2*n_z_ls_2) then ix2 = 2*n_z_ls_2
	iy1 = min(dom_2) - 3   & if (iy1  lt           0) then iy1 = 0
	iy2 = max(dom_2) + 3   & if (iy2  gt  2*n_z_ls_2) then iy2 = 2*n_z_ls_2
	s_lo_sy_h = s_lo_sy_h (ix1 : ix2,  iy1 : iy2)
    endif
;  Controle
    icon = 0
    if (icon eq 1) then begin
	i_choix_lobe = 1		; 1 sale,  2 synthetique
	j_sh_tv = 2 
;	n_size = 2 * n_z_ls_2 + 1
	n_size = 2 * n_z_ls_2 + 2
	if (i_choix_lobe  eq  1) then begin
	    s_lo_h = s_lo_sa_h
	    ch_lo_helio = 'sec lo sale helio'
	endif
	if (i_choix_lobe  eq  2) then begin
	    s_lo_h = s_lo_sy_h
	    ch_lo_helio = 'sec lobe synt helio'
	endif
;      Trace de la section
	if (j_sh_tv  eq  1) then begin
	    window, 2, xsize=n_size, ysize=n_size
	    loadct, 0
	    shade_surf, s_lo_h
	endif
	if (j_sh_tv  eq  2) then begin
	    window, 2, xsize=n_size, ysize=n_size
	    loadct, 13
	    tvscl, s_lo_h
	endif
	loadct, 0
	xyouts, 0.5, 0.95, /normal, alignment=0.5, charsize=1.3, ch_lo_helio
	help, ch_lo_helio
	stop
	    ; tvscl, congrid(im_int_p, 512, 512, cubic=-0.5)
	    ; tvscl, congrid(im_hel_p, 512, 512, cubic=-0.5)
    endif

; Section des lobes sale et synthetique pour une image  512*512  points
	; On part de  lobe_sale_hel_red  et  lobe_synt_hel_red  qui sont deja 
	; reduits et centr'es.
    n_trac = nint(512.0 * float(n_z_ls) / np)
;   help, n_trac
; Lobe sale
    lo_sale_hel_512 = congrid (lobe_sale_hel_red,  n_trac,  n_trac, cubic=-0.5)
    s_lo_sa_h_i = fltarr(n_trac,  n_trac)
    dom_sec = where(lo_sale_hel_512   ge   0.5 * max(lobe_sale_hel_red), ip)
	; on prend "ge" plutot que "gt" car il vaut mieux un lobe un peu trop
	; grand et aussi moins pixellis'e.
    s_lo_sa_h_i (dom_sec) = 1
;  Reduction a un champ ~ 2 fois plus etroit que celui du lobe
    n_trac_2 = nint(fac_red * n_trac / 2)
    ns1 = n_trac / 2  -  n_trac_2
    ns2 = n_trac / 2  +  n_trac_2
    s_lo_sa_h_i = s_lo_sa_h_i (ns1 : ns2,  ns1 : ns2)
; Lobe synthetique
    if (mode_clean  gt  0) then begin
	lo_synt_hel_512 = congrid (lobe_synt_hel_red,  n_trac,  n_trac, $
								cubic=-0.5)
	s_lo_sy_h_i = fltarr(n_trac,  n_trac)
	dom_sec = where(lo_synt_hel_512   ge   0.5 * max(lobe_synt_hel_red),ip)
	    ; on prend "ge" plutot que "gt" car il vaut mieux un lobe un peu 
	    ;   trop grand et aussi moins pixellis'e.
	s_lo_sy_h_i (dom_sec) = 1
;      Reduction a un champ ~ 2 fois plus etroit que celui du lobe
	n_trac_2 = nint(fac_red * n_trac / 2)
	ns1 = n_trac / 2  -  n_trac_2
	ns2 = n_trac / 2  +  n_trac_2
	s_lo_sy_h_i = s_lo_sy_h_i (ns1 : ns2,  ns1 : ns2)
    endif
; Reduction des sections des lobes heliographiques pour images  512*512  a des 
	; rectangles moins encombrants
;  Lobe sale
    som_sa_1 = reform(total(s_lo_sa_h_i, 2))	; somme sur 1er  indice
    som_sa_2 = reform(total(s_lo_sa_h_i, 1))	; somme sur 2eme indice
    dom_1 = where(som_sa_1  gt  0, ic1)
    dom_2 = where(som_sa_2  gt  0, ic2)
    ix1 = min(dom_1) - 10  & if (ix1  lt             0) then ix1 = 0
    ix2 = max(dom_1) + 10  & if (ix2  gt  2 * n_trac_2) then ix2 = 2 * n_trac_2
    iy1 = min(dom_2) - 10  & if (iy1  lt             0) then iy1 = 0
    iy2 = max(dom_2) + 10  & if (iy2  gt  2 * n_trac_2) then iy2 = 2 * n_trac_2
    s_lo_sa_h_i = s_lo_sa_h_i (ix1 : ix2,  iy1 : iy2)
;  Lobe synthetique
    if (mode_clean  eq  0) then  s_lo_sy_h_i = s_lo_sa_h_i
    if (mode_clean  gt  0) then begin
	som_sy_1 = reform(total(s_lo_sy_h_i, 2))    ; somme sur 1er  indice
	som_sy_2 = reform(total(s_lo_sy_h_i, 1))    ; somme sur 2eme indice
	dom_1 = where(som_sy_1  gt  0, ic1)
	dom_2 = where(som_sy_2  gt  0, ic2)
	ix1 = min(dom_1) - 10  & if (ix1  lt	0) then ix1 = 0
	ix2 = max(dom_1) + 10  & if (ix2  gt  2*n_trac_2) then ix2 = 2*n_trac_2
	iy1 = min(dom_2) - 10  & if (iy1  lt	       0) then iy1 = 0
	iy2 = max(dom_2) + 10  & if (iy2  gt  2*n_trac_2) then iy2 = 2*n_trac_2
	s_lo_sy_h_i = s_lo_sy_h_i (ix1 : ix2,  iy1 : iy2)
    endif
;  Controle
    icon = 0
    if (icon eq 1) then begin
	i_choix_lobe = 2		; 1 sale,  2 synthetique.
	j_sh_tv = 2 
	n_size = 2 * n_trac_2 + 1
	if (i_choix_lobe  eq  1) then begin
	    s_lo_h_i = s_lo_sa_h_i
	    ch_lobe = 'sec lobe sale helio'
	endif
	if (i_choix_lobe  eq  2) then begin
	    s_lo_h_i = s_lo_sy_h_i
	    ch_lobe = 'sec lobe synt helio'
	endif
;      Trace de la section
	if (j_sh_tv  eq  1) then begin
	    window, 2, xsize=n_size, ysize=n_size
	    loadct, 0
	    shade_surf, s_lo_h_i
	endif
	if (j_sh_tv  eq  2) then begin
	    window, 2, xsize=n_size, ysize=n_size
	    loadct, 13
	    tvscl, s_lo_h_i
	endif
	loadct, 0
	xyouts, 0.5, 0.95, /normal, alignment=0.5, charsize=1.3, ch_lobe
	help, ch_lobe
	stop
    endif

; Controle du flux de l'image interfero et du total de image heliographique,
	; et du niveau des images
    icon = 0
    if (icon eq 1) then begin
	flux_int    = total(im_int_p)	; flux interfero "propre". C'est vrai-
					;   ment un flux.
 	total_helio = total(im_hel_p)	; total image helio non normalisee. 
					;   Ce n'est qu'un total, et on ne lui
					;   donne pas le nom de flux
	print, ''
	print, 'RH_CALC_IMAGE_HELIO : apres clean, ' + $
		'AVANT les deux normalisations d''image helio :'
	print, '   comparaison des totaux des images interfero et helio ' + $
		'non limitee '
	print, '    total image interfero propre non periodisee :', flux_int
	print, '    ----------- helio     ------ non limitee    :', total_helio
;	stop
;      Traces images sales
	window, 0
	shade_surf, im_int_s
	xyouts, 0.5, 0.95, /normal, alignment=0.5, charsize=1.3, $ 
	    'Image interferometrique sale   (somme des pts = flux en sfu)'
	window, 1
	shade_surf, im_hel_s
	xyouts, 0.5, 0.95, /normal, alignment=0.5, charsize=1.3, $
	    'Image heliographique sale (somme des pts "ne" flux en sfu)'
;      Traces images propres
	window, 4
	shade_surf, im_int_p
	xyouts, 0.5, 0.95, /normal, alignment=0.5, charsize=1.3, $
	    'Image interferometrique propre   (somme des pts = flux en sfu)'
	window, 5
	shade_surf, im_hel_p
	xyouts, 0.5, 0.95, /normal, alignment=0.5, charsize=1.3, $
	    'Image heliographique propre   (somme des pts "ne" flux en sfu)'
;      Impression des maxima d'images
	print, 'Maxima d''images :   sales      propres'
	print, '   interfero', max(im_int_s), '    ',  max(im_int_p)  
	print, '   helio    ', max(im_hel_s), '    ',  max(im_hel_p)  
	stop
    endif
 m1: j_M=0.     ; i_affich_lobe
;----------------------------------------------------------
; Controle et comparaison des interpolations
    icon = 0
    if (icon eq 1) then begin
	window, 0
	shade_surf, im_hel_p, title='image heliographique propre'
	window, 1
	tvscl, congrid(im_hel_p, 512, 512, cubic=-0.5)
	stop
    endif			; fin de controle d'image "soleil rond".

; Controle et comparaison des interpolations donnant l'image helio propre
    icon = 0
    if (icon eq 1) then begin
	im_hel_p_lin = interpolate (im_int_p,  rg,  sg,  miss=0)
	rg2 = rg * 512. / np
	sg2 = sg * 512. / np	
;	window, 0
;	shade_surf, rg2, title='rg2'
;	window, 1
;	shade_surf, sg2, title='sg2'
;	stop
	im_int_p_512 = congrid(im_int_p, 512, 512, cubic=-0.5)
	im_hel_p_cub = interpolate (im_int_p_512,  rg2,  sg2,  miss=0)
	im_2d_512 = RH_INTER_FFT (im_int_p, 512)
	im_hel_p_fft = interpolate (im_int_p_512,  rg2,  sg2,  miss=0)
		; A REVOIR, SALADE DE NOTATIONS DANS CE CONTROLE
;     Visualisations
	window, 0
	shade_surf, im_hel_p_fft, title='im_hel_p_fft'
	window, 1
	shade_surf, im_2d_lin - im_2d_fft, title='im_2d_lin - im_2d_fft'
	window, 2
	shade_surf, im_2d_cub - im_2d_fft, title='im_2d_cub - im_2d_fft'
	stop
    endif			; fin de comparaison des interpolations.

	; Rappel :  im_hel_s  et  im_hel_p  sont les images heliographiques 
	;		sales et propres de np*np points.

    l_th_hel = interpolate (l_th,  rg,  sg,  miss=0)
	; Le lobe theorique n'intervenant dans la suite que pour des controles
	;   on ne fait pas d'interpolation couteuse en temps, qui doublerait
	;   presque le temps d'execution et on se contente de l'interpolation 
	;   lineaire.


; Limitation par "champ" (residu delouisien abandonne car inutile)
	; Rappel : champ met a zero l'image helio la ou rg et sg correspondent
	;   a des points exterieurs au champ interfero periodis'e. 
	; Du fait de "miss=0" dans le INTERPOLATE du calcul de l'image helio, 
	;   "champ" est inutile. On ne le conserve que par precaution (p ex 
	;   pour un calcul n'incluant pas miss=0)
;    im_hel_p = im_hel_p * champ	; image helio limitee.
;    im_hel_s = im_hel_s * champ	; image helio initiale limitee.

;    l_th_hel = l_th_hel * champ

; 
	
; NORMALISATIONS IMAGES HELIOGRAPHIQUES ET LOBES THEORIQUES HELIOGRAPHIQUES
; Premiere normalisation (pour une image heliographique exprimee en sfu/pixel
	; A ce stade les images restent en sfu/pixel et on normalise les images
	;   heliographiques de facon que la somme simple des points donne le 
	;   flux en sfu. 
	; L'interpolation pour passer de l'image interferometrique a l'image
	;   heliographique n'est qu'une interpolation et ne modifie pas les
	;   valeurs de l'image : on calcule seulement l'images en d'autres 
	;   points que sur la grille interferometrique. La maille heliographi-
	;   que etant differente de la maille interferometrique, le nombre de
	;   points sur l'extension de la source n'est pas le meme, et le total
	;   des images ne l'est pas non plus.
	; On fait ici une 1ere normalisation, telle que le total de l'image
	;   heliographique soit egal au total de l'image interferometrique.
	; Rem du 2 dec 2005 : le flux "interfero" est simplement la somme des
	;   points de l'image (admettant avoir resolu plus haut les problemes 
	;   de determination du niveau de base). L'angle solide domega_i de la
	;   maille interferometrique n'intervient donc pas dans le calcul du
	;   flux. Il me semble donc que c'est une erreur (dans RH_NORM_IM) de
	;   normaliser l'image heliographique en la multipliant par 
	;       domega_h / domega_i
	;   La normalisation doit se faire en exprimamt que la somme des points
	;   (le flux en sfu) est la meme pour les deux images. 
	;   On passera ensuite a la brillance exprimee en kelvins (grace au 
	;   facteur anorm retourn'e par RH_NORM_IM, et dont le calcul necessi-
	;   tera  d_omega_h
	; Rem du 8 mars 2006 : la notion de flux n'est valable que pour l'image
	;   non polar. Les images etant supposees correctement recalibrees, le
	;   facteur de normalisation doit etre le meme pour non polar et polar.
	;   S'il ne l'est pas, c'est sans doute par suite d'un mauvais calage 
	;   de zero, et le polar (qui n'a pas de soleil calme et dont la ligne
	;   de base est moins bien determinee) y est plus sensible.
	;   => On calcule le facteur de normalisation pour l'image non polar,
	;      et on le re-utilise au passage suivant pour l'image polar.
	;   Si i_polar=2 (pas de calcul d'image non polar) on calcule les fac-
	;   teurs de normalisation  anorm_1_s  et  anorm_1_p  a partir des
	;   images polar parce qu'il faut bien faire quelque chose, mais ca 
	;   sera sans doute mauvais.
    if (  (ch_polar  eq  'non polar')  or  $
	 ((ch_polar  eq      'polar') and (i_polar  eq  2)) ) then begin
	total_helio_s = total(im_hel_s)
	total_helio_p = total(im_hel_p)
	flux_int_p = total(im_int_p)
	anorm_1_s = flux_int_s / total_helio_s
	anorm_1_p = flux_int_p / total_helio_p
    endif
    flux_int_p = total(im_int_p)	; le flux est calcule aussi dans le cas
					; polar, mais n'a alors pas grand sens.
    anorm_lobe = 1.0 / total(l_th_hel)	; car  total(l_th_int) = 1. 
;	help, flux_int_s, flux_int_p, total_helio_s, total_helio_p, $
;		anorm_1_s, anorm_1_p
;	print, 'total(l_th_int) =', total(l_th) 
;	print, 'total(l_th_hel) =', total(l_th_hel) 
;	stop
;  normalisation proprement dite
    im_hel_s = anorm_1_s  * im_hel_s
    im_hel_p = anorm_1_p  * im_hel_p
    l_th_hel = anorm_lobe * l_th_hel
    flux_hel_s = flux_int_s	    ; la 1ere normalisation est faite pour ca.
    flux_hel_p = flux_int_p	    ; ---------------------------------------

    if version_solarsoft eq 0 then begin
;	print, '  Apres la 1ere normalisation :'
;	print, '    total(im_hel_s) =', total(im_hel_s)
;	print, '    total(image helio propre) =', total(im_hel_p)
;	print, '    total(l_th_hel) =', total(l_th_hel)
;	stop
;	print, format="(4x, 'total(image helio propre) =', f7.1, ' sfu')", $
;	    total(im_hel_p)	; utile a la rigueur.
    endif
	; - pour une image en unites de flux, le flux (en sfu) sera la 
	;     somme des points de l'image.
	; - pour une image en temperature de brillance l'image sera en Kelvins.
; Deuxieme normalisation, pour passer en kelvins
    if (t_b  eq  2) then begin
	RH_NORM_IM,  $					; dans  ~/rh_gene
	    champ_helio,  np,  domega_i,  freq,       $	; entree
	    anorm					; sortie
		; domega_i  angle solide sous-tendu par la maille interfero,
		;	     calcul'e par RH_MAILLE_3 et transmis depuis 
		;	     RH_DPATCHFITS_NRH. En principe inutilise apres 
		;	     le 2 dec 2005, mais laisse en argument par pre-
		;	     caution.
		; np, champ_helio  pour calculer l'angle solide de la maille
		;	     heliographique.
		; Le commentaire justifiant le calcul de anorm est en fin de 
		;   RH_NORM_IM.
	l_th_hel = anorm * l_th_hel		; lobe heliographique.
	im_hel_p = anorm * im_hel_p		; image ------------- propre.
	im_hel_s = anorm * im_hel_s		; ------------------- sale.
;	im_hel   = anorm * im_hel		; pour essais. C'est l'image
						;   helio propre avant mult par
						;   "champ" qui ne sert a rien.
    endif
;	print, ch_polar, ' : anorm_1_p =',  anorm_1_p
;	print, ch_polar, ' : anorm     =',  anorm
;	print, ch_polar, ' : anorm_1_p * anorm  =',  anorm_1_p * anorm
;	stop
    if version_solarsoft eq 0 then begin
;	print, '  Apres la 2eme normalisation (passage en kelvins) :'
	if (t_b eq 1) then $
	    ch_format = "(4x, 'max  (image helio propre) =', g10.3, " + $
							"' sfu/pixel')"
	if (t_b eq 2) then $
	    ch_format = "(4x, 'max  (image helio propre) =', g10.3, ' K')"
;	print, format=ch_format, max(im_hel_p)	; utile a la rigueur.
;	stop
    endif
; FIN DE NORMALISATION


; CALCUL DES FLUX (et non reconstitution, la composante continue etant recons-
	; tituee dans RH_MALC_IM_2D)
	; Remarque : 
	;	1) L'image est aliasee (ce que le clean peut reduire dans
	;	   une certaine mesure). De ce fait un objet compact contribue
	;	   a l'image non seulement a un "pic central" mais aussi dans 
	;	   les alias, qui contribuent au flux de facon importante. 
	;	2) Les harmoniques EW impairs existent seulement sur l'axe EW.
	;	   Par consequent les bordures N et S de l'image interferome-
	;	   trique (dont l'integrale est assujettie a etre nulle dans 
	;	   RH_MALC_IM_2D pour determiner le flux) ne sont pas nulles. 
	;	   Elles sont "gondolees" et presentent une extension deloca-
	;	   lisee en NS et modulee en EW qui n'est pas nulle mais qui 
	;	   est de moyenne nulle.
	;
	; La conclusion est qu'on ne peut pas obtenir le flux d'une source
	;   compacte en integrant l'image au voisinage du pic principal : le
	;   resultat peut etre sous-estime par un facteur allant jusqu'a envi-
	;   ron 2.
	; Ce qu'on peut faire en matiere de flux :
	;   - on se contente du flux total pour toute l'image, tel qu'il est 
	;	donn'e en sortie de RH_MALC_IM_2D par le total de l'image 
	;	interferometrique,
	;   - on fait un clean, qui "desaliase" considerablement et qui com-
	;	plete en NS le harm impairs de E0. Cela fait que la base de
	;	l'image d'une source compacte est plate loin du pic principal.
	;	Le flux devient donc "localise" et on peut integrer autour de
	;	ce pic sans grande erreur.
	;   - on utilise le flux de composante compacte fourni aussi en sortie
	;	de RH_MALC_IM_2D par l'extrapolation quadratique, et qui n'in-
	;	clut pas le flux de la base large. Toutefois cela ne marche
	;	que s'il y a une seule composante compacte (essais du 20 sept
	;	01), ou tout au moins si elle domine largement le reste.
	;	En outre si le soleil calme a un flux assez grand en comparai-
	;	son (p ex 5 fois) le flux compact est un peu sous estime (15% 
	;	pour 5 fois).
	;   - si on accepte une sous-estimation (env 2) du flux des sources
	;	compactes, ou si on ne s'interesse qu'a des variations relati-
	;	ves de 2 sources compactes, on peut deduire leur flux par une
	;	integration locale de l'image non cleanee autour de leurs pics
	;	principaux. On peut aussi corriger en multipliant par un 
	;	facteur 2 pour avoir leur flux absolu, ce qui est grossierement
	;	correct.	   
;    Flux_int_p = total(im_int_p)	; flux image interfero non etendue,
					;   brute ou cleanee. Si mode_clean=0
					;   on a pris im_int_p = im_int_s .
					; C'est le veritable flux (sfu), en
					;   pratique tres voisin de flux_int_s,
					;   qui est total(im_int_s), en sortie
					;   de RH_MALC_IM_2D.
;  Flux de l'image helio propre.
;    if (t_b  eq  1) then Flux_hel_p = total(im_hel_p)	
;    if (t_b  eq  2) then Flux_hel_p = total(im_hel_p) / anorm
					; rappel : anorm sortie de  RH_NORM_IM.
;  Flux du lobe (pour controles seulement)	
    F_l_th     = total (l_th)		; flux du lobe theorique interfero
    F_l_th_hel = total (l_th_hel)	; flux du lobe theorique interpole

; FIN DE CALCUL DES FLUX ET FIN DE RH_CALC_IMAGE_HELIO. 
; LA SUITE NE CONSISTE QU'EN CONTROLES

; Controle des flux		REVOIR LES NOTATIONS DE CE CONTROLE
    icon = 0
    if (icon eq 1) then begin
	if (t_b eq 1) then begin
	    im_2d_512    = congrid(im_2d   , 512, 512, cubic=-0.5)
	    im_int_p_512 = congrid(im_int_p, 512, 512, cubic=-0.5)
	    window, 0
	    tvscl, im_int_p_512
	    xyouts, 0.5, 0.97, /normal, alignment=0.5, charsize=ch_siz, $
				'image interferometrique finale'
	    window, 1
	    tvscl, im_2d_512
	    xyouts, 0.5, 0.97, /normal, alignment=0.5, charsize=ch_siz, $
				'image heliographique'
	    print, 'RH_CALC_IMAGE_HELIO apres :'
	    print, '  - reconstitution de comp continue dans RH_MALC_IM_2D , '
	    if (mode_clean eq 0) then  print, '  - sans clean '
	    if (mode_clean gt 0) then  print, '  - clean '
	    print, '  - normalisation image helio'
	    print, '  ATTENTION : le flux helio (somme de l''image helio) ' + $
		     'peut etre sous-estime'
	    print, '     si le champ helio < champ interfero'
	    print, ' '
	    ch_format = "('    Facteur de normalisation image helio = ', f5.2)"
	    print, format=ch_format, anorm
	    ch_format = "('    Total image interfero non etendue    =', " + $
							"f6.2, ' (sfu)')"
	    print, format=ch_format, Flux_int_p
	    ch_format = "('    Total image helio                    =', " + $
							"f6.2, ' (sfu)')"
	    print, format = ch_format, Flux_hel_p
	endif
	if (t_b eq 2) then begin
	    print, 'RH_CALC_IMAGE_HELIO : Somme des pts de l image helio ' + $
		'(en temp de brillance) =   ', Flux_hel_p
	endif
	stop
    endif

; Controle transformation du flux du lobe theorique.
    icon = 0
    if (icon eq 1) then begin
	ch_format = "('Freq: ',i3, '   Flux: interfero =', f5.2, " + $
		    "'    helio =', f5.2)"
	ifreq = fix(freq)
	print, format=ch_format, ifreq, Flux_int_p,  Flux_hel_p

	ch_format = "('rg min et max:', 2f8.1, 6x, " + $
		    "'sg min et max:', 2f8.1)"
	print,format=ch_format, min(rg), max(rg), min(sg), max(sg)

	ch_format = "('  Flux lobes theoriques:    interfero =', f5.2, " +$
		"'    helio =', f5.2)"
	print, format=ch_format,  F_l_th, F_l_th_hel
	stop
    endif
; Fin de controle.


; Etude de la localisation du flux de l'image interferometrique (on conserve le
	; centre d'image seul)
    icon = 0
    if (icon eq 1) then begin
	np2 = 20	; nbre de pts a conserver de part et d'autre du centre.
	im_i_c = fltarr (npi, npi)	; initialise image interfero centree
					;   mais mise a zero au dela de np2.
	im_i_c (npi/2-np2, npi/2-np2)  = $
				im_int_p(npi/2-np2 : npi/2+np2-1, $
				         npi/2-np2 : npi/2+np2-1)
	window, 0
	shade_surf, congrid(im_int_p, 512, 512, cubic=-0.5)
	window, 1
	shade_surf, congrid(im_i_c, 512, 512, cubic=-0.5)
	F_i_c = total (im_i_c)
	ifreq = fix(freq)
	ch_format = "('Freq: ',i3, '   Flux interfero =', f5.2, " + $
		"'    interfero limite a +-', i3, ' canaux  =', f5.2)"
	print, format=ch_format, ifreq, Flux_int_p, np2, F_i_c
	print, 'anorm =', anorm
	stop
    endif


; Controle images :
	; . interfero, 
	; . helio initiale (non cleanee) limitee par champ et ecretee,
	; . helio clean'ee,
	; et des flux.
	;
	; Controle utile dans RH_CLEAN ligne 2620 environ :
	; . objet reel soleil (simule par RH_SIMUL_HARM_N),
	; . image interferometrique sale, directement produite par le RH,
	; . ----------------------- propre lissee avec dom_c (en option). 

; Controle de l'image finale
    icon = 0			; utiliser plutot  i_stop_image  dans RH_DPNEW.
    if ((i_stop_image gt 0) or (icon eq 1)) then begin
;     Domaine annulaire de visualisation du limbe solaire
	    ; Le limbe interferometrique rest defini comme avant le 7 avril 04.
	    ;	Le trait reste epais.
	    ; Le limbe helographique (rond) et la croix sont directement defi-
	    ;	nis sur 512 points afin d'eviter un trait trop epais.
	if (i_pave_plein  eq  0) then  ch_pave_plein = $
						'Pave central non rempli.'
	if (i_pave_plein  eq  1) then  ch_pave_plein = 'Pave central rempli.'
;      Limbe heliographique avant 7 avril 04
;	d_o = shift(dist(np), -np/2, -np/2) * champ_helio / np		; en RS
;	dom_limb = where((d_o ge 0.985) and (d_o le 1.015))
;      Limbe heliographique
	limbe = fltarr(512, 512)
	r_sol = !pi/180*16/60				; rayon solaire (rad)
	champ_h_rad = champ_helio * r_sol
	x0 =0		& y0 =0		& ar0 = 1	; en Rs.
	ANNEAU, $					; fichier FASR_SUB.PRO
	    champ_h_rad, 512,  x0,  y0,  ar0,   $	; entree ar0 en radians
	    dom_limbe					; sortie
		; rappels :
		;  . champ_u a ete converti en radians plus haut. 
		;  . le rayon solaire est defini dans ANNEAU.
	f_l = 0.1
	limbe(dom_limbe) = f_l		; limite du limbe
	limbe(*      , 256:257) = f_l	; axe horizontal de la croix.
	limbe(256:257,       *) = f_l	; ---  vertical  -----------

	; niveau du controle final  (i_stop_image = 1  ou  icon = 1)

;     Ecretages separes des images
	    ; ECRETE ecrete separement ce qui est >0 et <0 .
	if (i_ecrete(0) eq 1) then $ 		; image interfero sale.
	    ECRETE, im_int_s, frac_ecr(0)
	if (i_ecrete(1) eq 1)    then $		; image interfero propre. 
	    ECRETE, im_int_p, frac_ecr(1)
	if (i_ecrete(2) eq 1)    then $	 	; image helio sale.
	    ECRETE, im_hel_s, frac_ecr(2)
	if (i_ecrete(3) eq 1)    then $		; image helio propre.
	    ECRETE, im_hel_p    , frac_ecr(3)	; icici

;     Calcul des maxima et minima des valeurs absolues d'images, utiles pour
	    ; le calcul de la crois et du limbe sur les images de controle
	max_abs_im_hel_s = max(abs(im_hel_s), min=min_abs_im_s, ip_s)
	max_abs_im_hel_p = max(abs(im_hel_p), min=min_abs_im_p, ip_p)	; icici

;	fac_dom = 0.01		; facteur pour visualiser dom_c et le limbe
	fac_dom = 0.1		; facteur pour visualiser dom_c et le limbe
	ch_format ="(f5.1, ' MHz ')"
	ch_freq = string(format=ch_format, freq)
	ch_freq_pol = ch_freq + ch_polar + ' '
;     Saut eventuel des traces des images interferometriques sales et propres
	if (i_stop_image eq 1) then i_saut = 2	; images interfero seules.
	if (i_stop_image eq 2) then i_saut = 1	; ------ helio     ------
	if (i_stop_image eq 3) then i_saut = 2	; ------ interfero + helio.
	i_sh_tv	= 2	; 1 shade_surf,  2 tvscl.
	if (i_sh_tv  eq  1) then xsiz = 640
	if (i_sh_tv  eq  2) then xsiz = 512
	n_win = 8	; numero de base pour les fenetres de sortie.
	i_color = 13	; couleur pout tvscl (0 BW, 13 rainbow). Shade_surf
			;   reste en noir et blanc.
	i_positif = 1	; pour avoir toujours un trace polar surtout >0, plus
			;   comprehensible a l'oeil.
;      Chgt de signe eventuel des images sur le trace, pour les rendre positi-
	    ; ves.
	if (i_positif  eq  0) then begin
	    ch_p_n_s = '(non changee de signe)'
	    ch_p_n_p = '(non changee de signe)'
	    i_p_n_s = 1
	    i_p_n_p = 1
	endif

	; niveau du controle final  (i_stop_image = 1  ou  icon = 1)

	if (i_positif  eq  1) then begin
	    max_im_int_s = max(im_int_s, min = min_im_int_s)
	    max_im_int_p = max(im_int_p, min = min_im_int_p)
	    if (abs(max_im_int_s)  gt  abs(min_im_int_s)) then i_p_n_s =  1
	    if (abs(max_im_int_s)  lt  abs(min_im_int_s)) then i_p_n_s = -1
	    if (abs(max_im_int_p)  gt  abs(min_im_int_p)) then i_p_n_p =  1
	    if (abs(max_im_int_p)  lt  abs(min_im_int_p)) then i_p_n_p = -1
		; i_p_n_s	drapeau de chagt de signe pour image sale
		; i_p_n_p	drapeau de chagt de signe pour image propre
	    if (i_p_n_s  eq   1) then  ch_p_n_s = '(non changee de signe). '
	    if (i_p_n_s  eq  -1) then  ch_p_n_s = '(changee de signe pour ' + $
								'lisibilite). '

	    if (i_p_n_p  eq   1) then  ch_p_n_p = '(non changee de signe). '
	    if (i_p_n_p  eq  -1) then  ch_p_n_p = '(changee de signe pour ' + $
								'lisibilite). '
;	      help, i_p_n_s, i_p_n_p, ch_p_n_s, ch_p_n_p
;	      stop
	endif
	if (i_saut eq 1) then goto, a10

	; niveau du controle final  (i_stop_image = 1  ou  icon = 1)

;     Images interfero sale et propre
	    ; les deux definitions suivantes sont recuperees des images propres
	    ;   pour lesquelles on tracait un limbe circulaire. Elles sont inu-
	    ;	tiles pour les images intefero, mais om se sert des adresses
	    ;	ip_s  et  ip_p  des maxima.
	max_abs_im_int_s = max(abs(im_int_s), min=min_abs_im_s, ip_s)
	max_abs_im_int_p = max(abs(im_int_p), min=min_abs_im_p, ip_p)

	trait = fltarr(npi, npi)
	trait(dom_a_i) = fac_dom * im_int_s(ip_s)
	im_int_s_512 = congrid(im_int_s  +  trait, 512, 512, cubic=-0.5) * $
								i_p_n_s
	trait = fltarr(npi, npi)
	trait(dom_a_i) = fac_dom * im_int_p(ip_p)
	im_int_p_512 = congrid(im_int_p  +  trait, 512, 512, cubic=-0.5) * $
								i_p_n_p

;      Incrustation de l'echelle de couleurs sur les images interfero
	    ; les max et min de l'echelle sont ceux de l'image tracee, eventu-
	    ;	ellement changee de signe.
	npc  = 201			; longueur de l'echelle de couleur.
	hpc  = 22 			; hauteur  -----------------------
	s_max = max(im_int_s_512, min=s_min)     & da_s = s_max - s_min
	p_max = max(im_int_p_512, min=p_min)     & da_p = p_max - p_min
;	    help, s_min, s_max, p_min, p_max
;	    stop
	ramp_n = findgen(npc) / (npc - 1)
	ramp_s = s_min + 1.05 * da_s * ramp_n
	ramp_p = p_min + 1.05 * da_p * ramp_n
		; il manque le haut de l'echelle si on ne prend pas 1.05
	band_s  = ramp_s # replicate(1, hpc)
	band_p  = ramp_p # replicate(1, hpc)
	band_s (*, hpc-5 : hpc-1) = 0    & band_p (*, hpc-5 : hpc-1) = 0
	    ; pour mettre azero le haut de la bande, ou on mettra les ticks de
	    ;  pourcentage.

	; niveau du controle final  (i_stop_image = 1  ou  icon = 1)

;      Incrustation d'une echelle de pourcentage dans la bande
	vms = 1.05 * s_max		& vmp = 1.05 * p_max
	a_m = 0.43				; bleu tres clair avec rainbow.
	for km = 0, 10  do begin		; boucle sur les ticks.
	    if (km eq 0) then begin		; elargiss a 2 pixels
		k1 = 20 * km
		k2 = 20 * km + 1
	    endif
	    if ( (km gt 0) and (km lt 10) ) then begin	; larg 3 pixels
		k1 = 20 * km - 1
		k2 = 20 * km + 1
	    endif
	    if (km eq 10) then begin	; elargissement a 2 pixels.
		k1 = 20 * km - 1
		k2 = 20 * km
	    endif
	    lm = 2			; longueur des ticks ("lt" interdit).
	    if ((km eq 0) or (km eq 5) or (km eq 10) ) then  lm = 4
	    band_s (k1 : k2, hpc - 1 - 4 : hpc - 1 - 4 + lm) = a_m * vms
	    band_p (k1 : k2, hpc - 1 - 4 : hpc - 1 - 4 + lm) = a_m * vmp
	endfor
	im_int_s_512 (0, 0) = band_s
	im_int_p_512 (0, 0) = band_p
;      Incrustation de la section des lobes interferometriques sale et propre
	    ; resultat du copier/coller des incrustations pour image propre,
	    ; inutilis'e au 2 mars 2006.
;	im_int_s_512 (0, 24) = s_lo_sa_h_i * a_m * vms  ; bleu-vert clair
;	if (mode_clean  eq  0) then $
;	    im_int_p_512 (0, 24) = s_lo_sa_h_i * a_m * vms
;	if (mode_clean  gt  0) then $
;	    im_int_p_512 (0, 24) = s_lo_sy_h_i * a_m * vmp
;      Fin des incrustations sur les images interferometriques.

	; niveau du controle final  (i_stop_image = 1  ou  icon = 1)

;      Trace image interferometrique sale
	window, n_win, xsize=xsiz, ysize=512		; USUEL n_win = 0.
	    ; rem : ysize est inoperant si xsize n'est pas defini.
	if (i_sh_tv eq 1) then begin
	    loadct, 0
	    shade_surf, im_int_s_512	; , az=-0  pour choix rotation
	endif
	if (i_sh_tv eq 2) then begin
	    loadct, i_color
	    tvscl, im_int_s_512
	endif
	    ; rappel : im_int_s_512  et  im_int_p_512  ont deja ete (eventuel-
	    ;	lement) changee de signe pour le trace.
	loadct, 0
	xyouts, 0.5, 0.97, /normal, alignment=0.5, charsize=ch_siz, $
	    ch_freq_pol + 'image interferometrique sale non periodisee'
	xyouts, 0.5, 0.93, /normal, alignment=0.5, charsize=ch_siz, $
	    ch_p_n_s + ch_pave_plein
;	print, 'Total(image interfero sale) =', total(im_int_s)  ; pour essai
;	print, 'Total(image helio     sale) =', total(im_hel_s)  ; pour essai
;      Trace image interferometrique propre 
	window, n_win + 1, xsize=xsiz, ysize=512
	if (i_sh_tv eq 1) then begin
	    loadct, 0
	    shade_surf, im_int_p_512 
	endif
	if (i_sh_tv eq 2) then begin
	    loadct, i_color
	    tvscl, im_int_p_512
	endif
	    ; rappel : im_int_s_512  et  im_int_p_512  ont deja ete (eventuel-
	    ;	lement) changee de signe pour le trace.
	loadct, 0
	xyouts, 0.5, 0.97, /normal, alignment=0.5, charsize=ch_siz, $
	    ch_freq_pol + 'image interferometrique propre non periodisee'
	xyouts, 0.5, 0.93, /normal, alignment=0.5, charsize=ch_siz, $
	    ch_p_n_s + ch_pave_plein
;     Impression des flux et des maxima d'images interferometriques non polar 
	    ; ou polar (utiles pour controle)
	ifreq = fix(freq)
	ch_format_1 = "(i3, ' MHz, flux " + ch_polar + " (sfu) :  coucou')"
	ch_format_2 = "(4x, 'compact =', f7.1, 6x, 'interfero =', f7.1, " + $
	    "'      helio =', f7.1)"
	ch_format_3 ="(4x, 'maxima abs(images) interero " + ch_polar + $
		"   sale :', e8.1, 3x, 'propre :', e8.1)"
	ch_format_4 = "(4x, 'rg min et max:', 2f8.1, 6x, " + $
			 "'sg min et max:', 2f8.1)"
	print, format=ch_format_1, nint(freq)
;	print, format=ch_format_2, flux_compact, Flux_int_p, Flux_hel_p
	print, format=ch_format_3, max_abs_im_int_s, max_abs_im_int_p
;	print, format=ch_format_4, min(rg), max(rg), min(sg), max(sg)

	if (i_saut eq 2) then goto, a11
a10:	J_M = 0			; certitude.

	; niveau du controle final  (i_stop_image = 1  ou  icon = 1)

;     Images helio sale et propre :
;      Interpolation a 512 et incrustation du limbe
	im_hel_s_limbe_512 = (congrid(im_hel_s, 512, 512, cubic=-0.5) + $
					max_abs_im_hel_s * limbe) * i_p_n_s
	im_hel_p_limbe_512 = (congrid(im_hel_p, 512, 512, cubic=-0.5) + $
					max_abs_im_hel_p * limbe) * i_p_n_p
;      Incrustation de l'echelle de couleurs en bas a gauche
	npc  = 201			; longueur de l'echelle de couleur.
	hpc  = 22 			; hauteur  -----------------------
	s_max = max(im_hel_s_limbe_512, min=s_min)    & da_s = s_max - s_min
	p_max = max(im_hel_p_limbe_512, min=p_min)    & da_p = p_max - p_min
	ramp_n = findgen(npc) / (npc - 1)
	ramp_s = s_min + 1.05 * da_s * ramp_n
	ramp_p = p_min + 1.05 * da_p * ramp_n
		; il manque le haut de l'echelle si on ne prend pas 1.05
	band_s  = ramp_s # replicate(1, hpc)
	band_p  = ramp_p # replicate(1, hpc)
	band_s (*, hpc-5 : hpc-1) = 0    & band_p (*, hpc-5 : hpc-1) = 0
;      Incrustation d'une echelle de pourcentage dans la bande
	vms = 1.05 * s_max		& vmp = 1.05 * p_max
	a_m = 0.43				; bleu tres clair avec rainbow.
	for km = 0, 10  do begin		; boucle sur les ticks.
	    if (km eq 0) then begin		; elargiss a 2 pixels
		k1 = 20 * km
		k2 = 20 * km + 1
	    endif
	    if ( (km gt 0) and (km lt 10) ) then begin	; larg 3 pixels
		k1 = 20 * km - 1
		k2 = 20 * km + 1
	    endif
	    if (km eq 10) then begin	; elargissement a 2 pixels.
		k1 = 20 * km - 1
		k2 = 20 * km
	    endif
	    lm = 2			; longueur des ticks ("lt" interdit).
	    if ((km eq 0) or (km eq 5) or (km eq 10) ) then  lm = 4
	    band_s (k1 : k2, hpc - 1 - 4 : hpc - 1 - 4 + lm) = a_m * vms
	    band_p (k1 : k2, hpc - 1 - 4 : hpc - 1 - 4 + lm) = a_m * vmp
	endfor
	im_hel_s_limbe_512 (0, 0) = band_s
	im_hel_p_limbe_512 (0, 0) = band_p

	; niveau du controle final  (i_stop_image = 1  ou  icon = 1)

;      Incrustation de la section des lobes heliographiques sale et propre
	im_hel_s_limbe_512 (0, 24) = s_lo_sa_h_i * a_m * vms  ; bleu-vert clair
	if (mode_clean  eq  0) then $
	    im_hel_p_limbe_512 (0, 24) = s_lo_sa_h_i * a_m * vms
	if (mode_clean  gt  0) then $
	    im_hel_p_limbe_512 (0, 24) = s_lo_sy_h_i * a_m * vmp
;      Trace image helio sale	
	if(i_saut eq 0) then  window, n_win + 2, xsize=xsiz, ysize=512
	if(i_saut eq 1) then  window, n_win    , xsize=xsiz, ysize=512
	if (i_sh_tv eq 1) then begin
	    loadct, 0
	    shade_surf, im_hel_s_limbe_512
	endif
	if (i_sh_tv eq 2) then begin
	    loadct, i_color
	    tvscl, im_hel_s_limbe_512
	endif
	    ; rappel : im_hel_s_limbe_512  et  im_hel_p_limbe 512  ont deja 
	    ;	ete (eventuellement) changee de signe pour le trace.
	loadct, 0
	xyouts, 0.5, 0.97, /normal, alignment=0.5, charsize=ch_siz, $
	    ch_freq_pol + 'image helio sale'
	xyouts, 0.5, 0.93, /normal, alignment=0.5, charsize=ch_siz, $
	    ch_p_n_p + ch_pave_plein

;     Trace image helio propre	
	if(i_saut eq 0) then window, n_win + 3, xsize=xsiz, ysize=512
	if(i_saut eq 1) then window, n_win + 1, xsize=xsiz, ysize=512
	if (i_sh_tv eq 1) then begin
	    loadct, 0
	    shade_surf, im_hel_p_limbe_512
	endif
	if (i_sh_tv eq 2) then begin
	    loadct, i_color
	    tvscl, im_hel_p_limbe_512
	endif
	    ; rappel : im_hel_s_limbe_512  et  im_hel_p_limbe 512  ont deja 
	    ;	ete (eventuellement) changee de signe pour le trace.
	loadct, 0
	xyouts, 0.5, 0.97, /normal, alignment=0.5, charsize=ch_siz, $
	    ch_freq_pol + 'image helio propre'	
					; anciennement " limitee par "champ" ".
	xyouts, 0.5, 0.93, /normal, alignment=0.5, charsize=ch_siz, $
	    ch_p_n_p + ch_pave_plein
a11:	J_M = 0		; certitude en ce monde incertain.

	if (t_b eq 1) then $
;		print, 'fact de norm image en sfu: anorm =', anorm
	if (t_b eq 2) then $
;		print, 'fact de norm image en  Tb: anorm =', anorm
;     Impression des flux et des maxima d'images non polar ou polar (utiles 
	    ; seulement pour controle)
	ifreq = fix(freq)
	ch_format_1 = "(i3, ' MHz, flux " + ch_polar + " (sfu) :')"
	ch_format_2 = "(4x, 'compact =', f7.1, 6x, 'interfero =', f7.1, " + $
	    "'      helio =', f7.1)"
	ch_format_3 ="(4x, 'maxima images helio " + ch_polar + "      sale" + $
	    " :', e9.1, 6x, 'propre :', e9.1)"
	ch_format_4 = "(4x, 'rg min et max:', 2f8.1, 6x, " + $
			 "'sg min et max:', 2f8.1)"
;	print, format=ch_format_1, nint(freq)
;	print, format=ch_format_2, flux_compact, Flux_int_p, Flux_hel_p
;	print, format=ch_format_3, max_abs_im_hel_s, max_abs_im_hel_p
;	print, format=ch_format_4, min(rg), max(rg), min(sg), max(sg)

;		stop
;		wset, 0 & plot, abs(harm_N)
;		stop

;     Ecriture des fichiers png des images affichees et du fichier postscript.
	    ; LES WRITE_PNG PRODUISANT DES MESSAGES INTEMPESTIFS A L'ECRAN, ON
	    ;   LES SAUTE
	ch_format ="(i3, '_MHz_')"
	ch_freq_0 = string(format=ch_format, nint(freq))
	i_saut_ecr = 1
		; ECRIRE LES IMAGES DANS UN Z-BUFFER AVANT DE SAISIR AVEC TVRD
	if (i_saut_ecr eq 1) then goto, a1
	i_tvrd = 1		; 0 ecriture des images normalisees a valmax 
				;     sans incrustations.
				; 1 ecriture traduction en byte de fenetre nor-
				;     isees a 136 avec incrustations.
	im_int_s_512 = congrid(im_int_s, 512, 512, cubic=-0.5) ; int sale
	im_int_p_512 = congrid(im_int_p, 512, 512, cubic=-0.5) ; int propre
	im_hel_s_512 = congrid(im_hel_s, 512, 512, cubic=-0.5) ; helio sale
	im_hel_p_512 = congrid(im_hel_p, 512, 512, cubic=-0.5) ; helio propre

;     Images interferometriques sale et propre
	if ((i_stop_image eq 1) or (i_stop_image eq 3)) then begin
	    n_fich = tete_fich + ch_freq_0 + 'int_sale.png'
;	    write_png, n_fich, bytscl(congrid(im_int_s, 512, 512,cubic=-0.5))
		; The BYTSCL function scales all values of Array that lie in 
		;  the range (Min  x  Max) into the range (0  x  Top). The 
		;  returned result has the same structure as the original para-
		;  meter and is of byte type.
		; Top is the maximum value of the scaled result. If not speci-
		;  fied 255 is used (ca devrait donc pouvoir ecreter, ce qui 
		;  est fait par ECRETE_IMAGE pour l'affichage).
	    if (i_tvrd eq 0) then  write_png, n_fich, im_int_s_512, r, g, b
	    if (i_tvrd eq 1) then  begin
		wset, 0			; interfero sale toujours en 0.
		write_png, n_fich, tvrd(), r, g, b
		    ; tvrd renvoie la traduction en byte de la fenetre graphi-
		    ; que, courante, cadree de 0 a 136 => perte de dynamique.
		    ; L'avantage est de conserver les incrustations.
	    endif
	    n_fich = tete_fich + ch_freq_0 + 'int_propre.png'
;	    write_gif, n_fich, bytscl(congrid(im_int_p, 512, 512, cubic=-0.5))
	    if (i_tvrd eq 0) then  write_png, n_fich, im_int_s_512, r, g, b
	    if (i_tvrd eq 1) then  begin
		wset, 1			; interfero propre toujours en 1.
		write_png, n_fich, tvrd(), r, g, b
	    endif
	endif

	; niveau du controle final  (i_stop_image = 1  ou  icon = 1)

;     Images heliographiques sale et propre
	if ((i_stop_image eq 2) or (i_stop_image eq 3)) then begin
	    n_fich = tete_fich + ch_freq_0 + 'helio_sale.png'
;	    write_gif, n_fich, bytscl(congrid(im_2d_s, 512, 512, cubic=-0.5))
	    if (i_tvrd eq 0) then  write_png, n_fich, im_2d_s_512, r, g, b
	    if (i_tvrd eq 1) then  begin
		if (i_stop_image eq 2) then wset, 0
		if (i_stop_image eq 3) then wset, 2
		write_png, n_fich, tvrd(), r, g, b
	    endif
	    n_fich = tete_fich + ch_freq_0 + 'helio_propre.png'
;	    write_gif, n_fich, bytscl(congrid(im_2d, 512, 512, cubic=-0.5))
	    if (i_tvrd eq 0) then  write_png, n_fich, im_2d_512, r, g, b
	    if (i_tvrd eq 1) then  begin
		if (i_stop_image eq 2) then wset, 1
		if (i_stop_image eq 3) then wset, 3
		write_png, n_fich, tvrd(), r, g, b
	    endif
	endif
;	print, 'RH_CALC_IMAGE_HELIO : fin'
a1:	J_M = 0			; reconfortante certitude.

	i_postscript = 0	; 1 pour changer  !d.name de 'x' en 'ps' , pour
				;   ecrire ci-dessous objet simule et image au
				;   choix (avec i_ch_im). 
				; Initialis'e ici car ce n'est qu'une verrue.

	if (i_postscript eq 0)  then stop
    endif; Fin de if ( (i_stop_image gt 0) or (icon eq 1)) .

; Incrustation de la section des lobes heliographiques sale ou propre (c'est a
	    ; dire synthetique) dans les images sales et propres repassees a 
	    ; RH_DPATCHFITS_NRH  pour ecriture fits.
    if (i_affich_lobe  eq  1) then begin
	im_hel_max = max(im_hel_s)
	    ; recalcul'e ici car il y a pu avpir une ecretage.
	    ; a_m defini plus haut egal a  0.43  dans le controle final pour du
	    ;   bleu-vert clair.
	a_m = 0.43
	im_hel_s (0, 2) = s_lo_sa_h * a_m * im_hel_max
	if (mode_clean  eq  0) then  im_hel_p = im_hel_s
	if (mode_clean  gt  0) then begin 
	im_hel_max = max(im_hel_p)
	im_hel_p (0, 2) = s_lo_sy_h * a_m * im_hel_max
	endif
;     Controle
	icon = 0
	    if (icon  eq  1) then begin
	    window, 12, xsize=np, ysize=np
	    loadct, 13
	    tvscl, im_hel_s
	    window, 13, xsize=np, ysize=np
	    loadct, 13
	    tvscl, im_hel_p
	    stop
	    endif	; fin de controle
    endif		; fin d'incrustation eventuelle.

; Comparaison image sale avec objet simul'e et image helio propre, ces deux
	; derniers liss'es (dans RH_CLEAN) par le meme filtre final
    icon = 0
    if ((i_simulation  gt  0) and (icon  eq  1)) then begin
	i_sh_tv = 1			; 1 shade_surf,  2 tvscl
;     Domaine annulaire de visualisation du limbe solaire
	d_o = shift(dist(np), -np/2, -np/2) * champ_helio / np		; en RS
	dom_limb = where((d_o ge 0.985) and (d_o le 1.015))
	    ; Rem : visualiser le limbe est pei utile en simulation.
;     Objet simul'e liss'e
	window, 0
	if (i_sh_tv eq 1) then $
	    shade_surf, congrid(objet_simule_lisse, 512, 512, cubic=-0.5)
	if (i_sh_tv eq 2) then $
	    tvscl     , congrid(objet_simule_lisse, 512, 512, cubic=-0.5)
	xyouts, 0.5, 0.97, /normal, alignment=0.5, charsize=ch_siz, $
		'objet simule lisse avec meme filtre que l''image propre'
;     Image helio sale
	window, 1, xsize=512, ysize=ysiz
	fac_dom = 0.3		; facteur pour visualiser dom_c et le limbe
	max_im = max(abs(im_2d_s), ip)
	trait = fltarr(np, np)		; tours du disque solaire et de dom_c.
	a_trait = fac_dom * im_2d_s(ip)
	trait(dom_a_h)  = a_trait
	trait(dom_limb) = a_trait
	if (i_sh_tv eq 1) then $
	    shade_surf, congrid(im_2d_s + 0 * trait, 512, 512, cubic=-0.5)
	if (i_sh_tv eq 2) then $
	    tvscl     , congrid(im_2d_s + 0 * trait, 512, 512, cubic=-0.5)
	xyouts, 0.4, 0.97, /normal, alignment=0.5, charsize=ch_siz, $
		'image helio sale'
;     Image helio propre lissee par le filtre final
	window, 2, xsize=512, ysize=ysiz
	max_im = max(abs(im_2d), ip)
	trait = fltarr(np, np)		; tours du disque solaire et de dom_c.
	a_trait = fac_dom * im_2d(ip)
	trait(dom_a_h)  = a_trait
	trait(dom_limb) = a_trait
	if (i_sh_tv eq 1) then $
	    shade_surf, congrid(im_2d + 0 * trait, 512, 512, cubic=-0.5)
	if (i_sh_tv eq 2) then $
	    tvscl     , congrid(im_2d + 0 * trait, 512, 512, cubic=-0.5)
	xyouts, 0.4, 0.97, /normal, alignment=0.5, charsize=ch_siz, $ 
		'image helio propre lissee par le filtre final'	
					; anciennement " limitee par "champ" ".
	stop
    endif

; Ecriture de fichier postscript (pour transparents) pour :
	; . objet simule,
	; ' image au choix.
;  Choix de l'image a ecrire
    if (n_elements(i_postscript) eq 1) then begin
	    ; test existence de i_postscript, qui n'est defini que dans le con-
	    ;	trole des images (
      if (i_postscript eq 1) then begin
	    ; n_elements vaut 0 si la variable n'est pas definie (piege et
	    ;    inutilite du keyword_set).
	ch_titre = '!3 Image interferometrique'
	i_ch_im = 1		; 1 image interfero sale
				; 2 --------------- propre
				; 3 ----- helio     sale
				; 4 -----------     propre
	trait = 0		; supprimer ou non le trait
	objet =  congrid(objet_simule, 512, 512, cubic=-0.5)
	if (i_ch_im eq 1) then $
		image = congrid(im_int_s + trait, 512, 512, cubic=-0.5)
	if (i_ch_im eq 2) then $
		image = congrid(im_int_p + trait, 512, 512, cubic=-0.5)
	if (i_ch_im eq 3) then $
		image = congrid(im_2d_s  + trait, 512, 512, cubic=-0.5)
	if (i_ch_im eq 4) then $
		image = congrid(im_2d      + trait, 512, 512, cubic=-0.5)
	set_plot, 'ps'			; change !d.name de 'x' en 'ps'
	dx_p = 21    & dy_p = 29		; dimensions du papier.
	dx_f = 20.5  & dy_f = 29		; --------- de la fenetre.
;     Cadrage approximatif de la fenetre dans la feuille.
	x_off = 0.5 * (dx_p - dx_f) 
	y_off = 0.5 * (dy_p - dy_f)
		; pour avoir une fenetre cadree dans la page. icici
	device, filename = 'txx.ps', /portrait, $
	    xsize   =  dx_f,  ysize   = dy_f, $		; taille fenetre en cm.
	    xoffset =  x_off,  yoffset = y_off		; pos. coin inf gauche

	dy_titre = 0.05			; hauteur fenetre de titre (coord nor-
					;   malisees).
	h_norm = dx_f / dy_f		; hauteur normalisee d'une fenetre car-
					;   ree de meme largeur que le papier.
	h_res = 1 - dy_titre - h_norm	; hauteur restant inutilisee.
;     Ecriture titre du trace shade_surf de l'objet
	!p.multi = [0, 1, 2]		; !p.multi(0)=0 pour changer de page.
	!p.region = [0, 1 - dy_titre,   1, 1]
	plot, [0], [0], xstyle=5, ystyle=5, xrange=[0, 1], yrange=[0, 1]
	xyouts, 0.5, 0.98, /normal, alignment=0.5, charsize=3, charthick=5, $
		'Objet'
;     Ecriture trace shade_surf de l'objet
	!p.multi = [1, 1, 2]
;	!p.region = [0, 1 - dy_titre - 0.5 * h_res - h_norm,   $
;				1, 1 - dy_titre - 0.5 * h_res]
	!p.region = [0, 1 - dy_titre - h_norm,   1, 1 - dy_titre] 
	print, !p.region
	!x.margin = [2, 2]			; en dimension de caracteres.
	shade_surf, objet
;     Ecriture titre du trace tvscl de l'objet
	!p.multi = [0, 1, 2]		; !p.multi(0)=0 pour changer de page.
	!p.region = [0, 1 - dy_titre,   1, 1]
	plot, [0], [0], xstyle=5, ystyle=5, xrange=[0, 1], yrange=[0, 1]
	xyouts, 0.5, 0.98, /normal, alignment=0.5, charsize=3, charthick=5, $
		'Objet'
;     Ecriture trace tvscl de l'objet
	!p.multi = [1, 1, 2]
;	!p.region = [0, 1 - dy_titre - 0.5 * h_res - h_norm,   $
;				1, 1 - dy_titre - 0.5 * h_res] 
	!p.region = [0, 1 - dy_titre - h_norm,   1, 1 - dy_titre] 
	print, !p.region
	!x.margin = [2, 2]			; en dimension de caracteres.
	tvscl, objet
	
;     Ecriture titre du trace shade_surf image
	!p.multi = [0, 1, 2]		; !p.multi(0)=0 pour changer de page.
	!p.region = [0, 1 - dy_titre,   1, 1]
	plot, [0], [0], xstyle=5, ystyle=5, xrange=[0, 1], yrange=[0, 1]
	xyouts, 0.5, 0.98, /normal, alignment=0.5, charsize=3, charthick=5, $
		ch_titre
;     Ecriture trace shade_surf image
	!p.multi = [1, 1, 2]
;	!p.region = [0, 1 - dy_titre - 0.5 * h_res - h_norm,   $
;				1, 1 - dy_titre - 0.5 * h_res] 
	!p.region = [0, 1 - dy_titre - h_norm,   1, 1 - dy_titre] 
	!x.margin = [2, 2]			; en dimension de caracteres.
	shade_surf, image
;     Ecriture titre du trace tvscl image
	!p.multi = [0, 1, 2]		; !p.multi(0)=0 pour changer de page.
	!p.region = [0, 1 - dy_titre,   1, 1]
	plot, [0], [0], xstyle=5, ystyle=5, xrange=[0, 1], yrange=[0, 1]
	xyouts, 0.5, 0.98, /normal, alignment=0.5, charsize=3, charthick=5, $
		ch_titre
;     Ecriture trace tvscl image
	!p.multi = [1, 1, 2]
;	!p.region = [0, 1 - dy_titre - 0.5 * h_res - h_norm,   $
;				1, 1 - dy_titre - 0.5 * h_res] 
	!p.region = [0, 1 - dy_titre - h_norm,   1, 1 - dy_titre] 
	!x.margin = [2, 2]			; en dimension de caracteres.
	tvscl, image
	
	device, /close
	stop
      endif		; fin d'ecriture de dpxxx.ps
    endif		; fin if sur test existence de i_postscript.

a2: J_M = 0		; fin du saut general pour  harm_N  identiquement nul

    end			; fin de RH_CALC_IMAGE_HELIO



;pro draw_circle,x0,y0,radius,npts=npts,device=device,data=data,psym=psym,  $
;                        linestyle=linestyle,color=color,thick=thick,       $
;                        fill=fill,noclip=noclip
;+
;  Name:
;    draw_circle
;  Purpose:
;    Draw a circle.
;  Calling Sequence:
;    draw_circle, x0, y0, radius [, npts=npts]
;  Input Parameters:
;    x0 =  Central x coordinate
;    y0 =  Central y coordinate
;    radius = Radius
;
;  Optional input keywords:
;    Npts   = Number of points (default = 10000)
;    device = If set, plots will assume device coordinates
;    data   = If set, plots will assume data   coordinates
;    psym   = Plot symbol
;    color  = Color
;    linestyle = Line style
;    fill   = If set, call poly_fill
;    noclip = If noclip=0, circles will be clipped (read the IDL
;               manual on plots to find out why its this way).
;

